/***************************************************************************/
/*   File name   : GTR.h                                                   */
/*   Description : Full Text Search Engine GTR interface                   */
/***************************************************************************/
/* Licensed Materials - Property of IBM                                    */
/* Global Text Retrieval (GTR)                                             */
/* (C) Copyright IBM Corporation 1996, 2001.                               */
/* All rights reserved.                                                    */
/*                                                                         */
/* US Government Users Restricted Rights - Use, duplication or             */
/* disclosure restricted by GSA ADP Schedule Contract with                 */
/* IBM Corporation.                                                        */
/***************************************************************************/
#ifndef GTR
#define GTR
#ifndef GTRtype
#include "GTRtype.h"
#endif

#ifdef  __cplusplus                                     /* @3-043D *//* @4-204A */
/* #if defined __cplusplus && !defined GTR_CPP  */      /* @3-043A *//* @4-204D */
  extern "C" {
#endif

#if defined _WIN32 || defined __WINDOWS__               /* @3-019A */
    #define GTR_LINK __cdecl                            /* @3-019A */
#elif defined __OS2__                                   /* @3-019A */
    #define GTR_LINK _Optlink                           /* @3-019A */
#else                                                   /* @3-019A */
    #define GTR_LINK                                    /* @3-019A */
#endif                                                  /* @3-019A */


/* #if defined __IBMC__ || defined __IBMCPP__           @3-019D */
/*    #define GTR_LINK _Optlink                         @3-019D */
/* #else                                                @3-019D */
/*    #if defined _WIN32                                @3-019D */
/*        #define GTR_LINK __cdecl                      @3-019D */
/*    #else                                             @3-019D */
/*        #define GTR_LINK                              @3-019D */
/*    #endif                                            @3-019D */
/* #endif                                               @3-019D */

/**************************************************************************/
/*     Function Interface                                                 */
/**************************************************************************/
/*------------------------------------------------------------------------*/
/*   1. Create index / Add to index                                       */
/*------------------------------------------------------------------------*/
/*   Calling sequence :                                                   */
/*                                                                        */
/*        ---  Initialization  ---                           ---          */
/*        GTRcreateIndexInit( .. , &GTR_status);              |           */
/*        if (GTR_status.retcode != 0) goto laberr;           |           */
/*                                                            |           */
/*        for (DocNO = FirstDocNO; Until last doc; ++DocNO) { |           */
/*                                                            |           */
/*          Read one document on memory                       |           */
/*                                                            |           */
/*          --- Process of one document  ---                  |           */
/*          GTRcreateIndexOneDoc( .. , &GTR_status);          |           */
/*                         or                                 |           */
/*          GTRcreateItemIndexOneDoc( .. , &GTR_status);      |           */
/*          if (GTR_status.retcode != 0) goto laberr;         |           */
/*                                                            |           */
/*        }                                                   |           */
/*        --- Create index with temporary filename ---        |           */
/*        GTRcreateIndexLast( .. , &GTR_status);              |           */
/*        if (GTR_status.retcode != 0) goto laberr;           |           */
/*                                                            |           */
/*        --- Completion of index creation ---                |  ---      */
/*        GTRcreateIndexTerm( .. , &GTR_status);              |   |       */
/*        if (GTR_status.retcode != 0) goto laberr;           |   |       */
/*                                                           --- ---      */
/*      laberr:                                             CR,E SE,E     */
/*        GTRgetMessage(&GTR_status, .. );                                */
/*                                                                        */
/*------------------------------------------------------------------------*/
/*-----                       Initializaion                          -----*/
void GTR_LINK GTRcreateIndexInit(
     void           **cwpp,                   /*                       O */
     const BYTE1    index_name[],             /* Index name            I */
     const BYTE1    index_directory[],        /* Index directory       I */
     const BYTE1    work_directory[],         /* Work derectory        I */
     BYTE1          operation,                /*                       I */
                    /*--  Create new index (existing one is replaced)  --*/
                    #define CREATE_INDEX_NEW 'N'
                    /*--  Add to the supplemental index if index exists--*/
                    /*--  Otherwise, the index is created.             --*/
                    #define CREATE_INDEX_ADD 'A'
     BYTE1          batch_or_online,          /* Batch | Online        I */
                                 /*--  As a batch job  --*/
                                 #define BATCH_CREATE   'B'
                                 /*--  As a online job --*/
                                 #define ONLINE_CREATE  'O'
     GTRHCREATEPARM *create_parameter_ptr,    /*                       I */
     GTRSYSPARM     *system_parameter_ptr,    /* System parameters     I */
                                              /* NULL : Default is used  */
     INT4           *current_maximum_docNO,   /* Max doc No. in index  O */
     GTRSTATUS      *gtr_status);             /*                       O */
     /*--- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --*/
     /* (NOTE) How to specify the system parameters                      */
     /*          1. Via parameter of GTRcreateIndexInit                  */
     /*          2. Via GTRSYS file in the current path                  */
     /*        1 is preferred to 2.                                      */
     /*        If neither is specified, the default values are used.     */
     /*--- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --*/


/*-----                     Process one document                    -----*/
void GTR_LINK GTRcreateIndexOneDoc(
     void          *work_area_ptr,   /* Ptr. set by GTRcreateIndexInit   I */
     UBYTE1        *doc_on_memory,   /* Ptr. to one document contents I    */
     INT4           doc_length,      /* Length of document contents      I */
     INT4           doc_No,          /* Document No. assigned to the doc I */
     INT4          *position,        /* Position No. of the top char.    I */
                                     /* Position No. assigned to last ch O */
     BOUNDARYINFO  *block_end_info_ptr, /* Information on boundaries     I */
     GTRSTATUS     *GTR_status);     /* Status codes                     O */

/*----- Add or Update one document                      @4-002A Start -----*/
void GTR_LINK GTRcreateIndexOneDocExt(
     void          *work_area_ptr,   /* Ptr. set by GTRcreateIndexInit   I */
     UBYTE1        *doc_on_memory,   /* Ptr. to one document contents    I */
     INT4           doc_length,      /* Length of document contents      I */
     INT4           doc_No,          /* Document No. assigned to the doc I */
     INT4          *position,        /* Position No. of the top char.    I */
                                     /* Position No. assigned to last ch O */
     BOUNDARYINFO  *block_end_info_ptr, /* Information on boundaries     I */
     GTRONEDOCINFO *pGTRDocInfo,     /* Document info           @4-002A  I */
     GTRSTATUS     *GTR_status);     /* Status codes                     O */

/*-----         Delete information on specified documents           -----*/
void GTR_LINK  GTRdeleteDocs(
     void *work_area_ptr,            /* Ptr. set by GTRcreateIndexInit  I */
     UINT4 number_of_delete_doc_NO,   /* Number of document numbers      I */
     UINT4 *delete_doc_NO,   /* Pointer to the array of document numbers I */
     void *pReserved,                 /* (reserved) Specify NULL         I */
     GTRSTATUS *GTR_status            /* Status codes                    O */
     );
/*-----                                                   @4-002A End -----*/

/*-----        Create the target index with temporary filenames     -----*/
/* #define GTRcreateIndexLast GTR_createIndexLast */          /* @4-002D */
void GTR_LINK  GTRcreateIndexLast(
      void *work_area_ptr,           /* Ptr. set by GTRcreateIndexInit I */
      GTRSTATUS *GTR_status          /* Status codes                   O */
      );

/*-----   Completion -  rename the temporary filenames to target ones ---*/
/* #define GTRcreateIndexTerm GTR_createIndexTerm */          /* @4-002D */
void GTR_LINK  GTRcreateIndexTerm(
      void       *work_area_ptr,     /* Ptr. set by GTRcreateIndexTerm I */
      BYTE1      go_or_cancel,       /* GTR_GO | GTR_CANCEL            I */
                        /*--  Complete indexing             --*/
                        #define GO_CREATE     'G'
                        /*--  Cancel indexing               --*/
                        #define CANCEL_CREATE 'C'
      GTRSTATUS *GTR_status          /* Status codes                   O */
      );

/*-----             Process one document for Item Index             -----*/
void GTR_LINK GTRcreateItemIndexOneDoc(
     void       *work_area_ptr,      /* Ptr. set by GTRcreateIndexInit I */
     INT4       document_number,     /* Document Number (ID)           I */
     INT4       nubmer_of_item_data, /* Number of Item data            I */
     REGISTITEM item_data[],         /* Item data                      I */
     GTRSTATUS  *GTR_status          /* Status codes                   O */
     );

/*-----             Add or Update for Item Index      @4-002A Start -----*/
void GTR_LINK GTRcreateItemIndexOneDocExt(
     void       *work_area_ptr,      /* Ptr. set by GTRcreateIndexInit I */
     INT4       document_number,     /* Document Number (ID)           I */
     INT4       nubmer_of_item_data, /* Number of Item data            I */
     REGISTITEM item_data[],         /* Item data                      I */
     GTRONEDOCINFO *pGTRDocInfo,     /* Document info          @4-002A I */
     GTRSTATUS  *GTR_status          /* Status codes                   O */
     );
/*-----             Add or Update for Item Index      @4-002A End   -----*/

/*------------------------------------------------------------------------*/
/*   2. Search                                                            */
/*------------------------------------------------------------------------*/
/*   2.1  Search                                                          */
/*------------------------------------------------------------------------*/
/*   Calling sequence :                                                   */
/*                                                                        */
/*        GTRSEARCH_HANDLE s_handle = NULL;                               */
/*        GTRSTATUS        GTR_status;                                    */
/*                                                                        */
/*        for ( ; ; ) {                                                   */
/*          GTRsearchIndex(&s_handle, .. , &GTR_status);                  */
/*          if (GTR_status.retcode != 0) {                                */
/*            -----  Error process  -----                                 */
/*            GTRgetMessage(.., &GTR_status, ..);                         */
/*          }                                                             */
/*        }                                                               */
/*                                                                        */
/*        GTRsearchIndex_Release(s_handle, &GTR_status);                  */
/*                                                                        */
/*------------------------------------------------------------------------*/
/*-----   Search Text Index or Item Index  -----*/
void GTR_LINK GTRsearchIndex(
     GTRSEARCH_HANDLE *ptr_to_search_handle,                     /*    IO */
                          /* Ptr to Handle set by previous call as input  */
                          /*   NULL : Handle is not needed.               */
                          /*          (ex.) GTRsearchIndex(NULL, ...);    */
                          /*   Handle must be NULL at first call.         */
                          /*          (ex.) handle = NULL;                */
                          /*                GTRsearchIndex(&handle, ...); */
                          /* Ptr to Handle is set               as output */
     const BYTE1      index_name[],   /* Index name        I */     /* @4-220A */
     const BYTE1      index_directory[],  /* Index Directory   I */ /* @4-220A */
     const BYTE1      output_directory[], /* Where workfile      */ /* @4-220A */
     /* const BYTE1      index_name[IDXNAMELENMAX],    */ /* Index name        I */ /* @4-220D */
     /* const BYTE1      index_directory[DIRLENMAX],   */ /* Index Directory   I */ /* @4-220D */
     /* const BYTE1      output_directory[DIRLENMAX],  */ /* Where workfile      */ /* @4-220D */
                                                   /*   can be written  I */
     GTRSEARCH_COND   search_condition,            /* Search Condition  I */
     GTRSEARCH_RANK   *ptr_to_ranking_parameter,   /* Ranking Parameter I */
     void             *reserved_ptr1,              /* Specify NULL      I */
     void             *reserved_ptr2,              /* Specify NULL      I */
     GTRSEARCH_RES    *ptr_to_search_result_parm,  /* Search Result    IO */
     GTRSTATUS        *GTR_status
     );

/*-----   Release Search Handle  -----*/
void GTR_LINK GTRsearchIndex_Release(
     GTRSEARCH_HANDLE *ptr_to_search_handle,/* Search handle to be released I */
     GTRSTATUS        *GTR_status
     );


/*------------------------------------------------------------------------*/
/*   2.2  Get Search Result (used only when res_type == GTR_RES_HANDLE)   */
/*------------------------------------------------------------------------*/
/*   Calling sequence :                                                   */
/*                                                                        */
/*        GTRSEARCH_RES    result_parm;                                   */
/*        GTRRESULT_HANDLE resHandle;                                     */
/*                                                                        */
/*        result_parm.res_type = GTR_RES_HANDLE;                          */
/*                                                                        */
/*        GTRsearchIndex( .., &result_parm, .., &GTR_status);             */
/*        resHandle = result_parm.res_handle;                             */
/*        |                                                               */
/*        |  GTRsearchIndex_OpenResult(resHandle, &GTR_status);           */
/*        |  |                                                            */
/*        |  |   GTRsearchIndex_GetResult(resHandle, .., &GTR_status);    */
/*        |  |                                                            */
/*        |  GTRsearchIndex_CloseResult(resHandle, &GTR_status);          */
/*        |                                                               */
/*        GTRsearchIndex_ReleaseResult(resHandle, &GTR_status);           */
/*                                                                        */
/*                                                                        */
/*------------------------------------------------------------------------*/
typedef INT4 GTR_DOC_OFFSET;
typedef INT4 GTR_OCC_OFFSET;


/*-----  Open Result Handle (used only when res_type = GTR_RES_HANDLE) ---*/
void GTR_LINK GTRsearchIndex_OpenResult(
     GTRRESULT_HANDLE result_handle,       /* Result Handle             I */
     void             *rsv_ptr,            /* (reserved)                  */
     GTRSTATUS        *GTR_status
     );


/*-----  Get Result         (used only when res_type = GTR_RES_HANDLE) ---*/
GTR_OCC_OFFSET GTR_LINK GTRsearchIndex_GetResult(
     GTRRESULT_HANDLE result_handle, /* Result Handle                   I */
     GTR_OCC_OFFSET   occ_offset,    /* First OCC No to be read(from 0) I */
                                     /* -1: Currenct OCC No.              */
     INT4             number_to_be_read, /* Number of OCC to be read    I */
     OCC              occurrences[], /* OCC to be read                  O */
                      /* (NOTE) User program must allocate and release    */
                      /*        area for OCC.                             */
     void             *rsv_ptr1,     /* (reserved)                        */
     void             *rsv_ptr2,     /* (reserved)                        */
     GTRSTATUS        *GTR_status
     );
     /*---  Returned is the occurrence offset next to the last read  ---*/

/*-----  Look Result by Doc (used only when res_type = GTR_RES_HANDLE) ---*/
GTR_DOC_OFFSET GTR_LINK GTRsearchIndex_LookResult_Doc(
     GTRRESULT_HANDLE result_handle,  /* Result Handle                  I */
     GTR_DOC_OFFSET   doc_offset,     /* Offset in result of the doc to I */
                                      /* be looked                        */
                                      /* -1: Currenct document            */
     OCC              *occurrences[], /* Ptr to OCC is set              O */
                      /* All OCCs of the document to be specified by      */
                      /* doc_offset                                       */
                      /* (NOTE) GTR allocates area for OCC.               */
                      /*        The area is released by GTR at next call. */
     INT4             *number_looked, /* Number of OCC of the document  O */
     FLOAT4           *score,         /* Score of ranking               O */
     void             *rsv_ptr1,      /* (reserved)                       */
     GTRSTATUS        *GTR_status
     );
     /*---  Returned is the document offset next to the read one ---*/

/*-----  Close Result       (used only when res_type = GTR_RES_HANDLE) ---*/
void GTR_LINK GTRsearchIndex_CloseResult(
     GTRRESULT_HANDLE result_handle,  /* Result Handle                  I */
     void             *rsv_ptr,       /* (reserved)                       */
     GTRSTATUS        *GTR_status
     );


/*-----  Release Result     (used only when res_type = GTR_RES_HANDLE) ---*/
void GTR_LINK GTRsearchIndex_ReleaseResult(
     GTRRESULT_HANDLE result_handle,  /* Result Handle                  I */
     GTRSTATUS        *GTR_status
     );

/*------------------------------------------------------------------------*/
/*---------------------      For GTRV2 Users     -------------------------*/
/*------------------------------------------------------------------------*/
/*-----                      Simple Search                           -----*/
void GTR_LINK GTRsimpleSearch(
     UBYTE1        query[],           /* Search terms delimited by      I */
                                      /* blank characters, connected by   */
                                      /* a operator in simple_option.     */
     SIMPLERTRVOPT simple_option,     /* Search parameters              I */
     BYTE1         index_name[],      /* Index name                     I */
     BYTE1         index_path[],      /* Index directory                I */
     BYTE1         output_fname[],    /* Name of result file            I */
     BYTE1         output_path[],     /* Result file directory          I */
     UBYTE1        codepage,          /* Codepage (same as codepage     I */
                                      /*              of GTRHCREATEPARM I */
     RTRVRANK     *rank_info_ptr,     /* Parameters for ranking         I */
     INT4         *answer_doc_number, /* Number of documents of result  O */
     INT4         *answer_occ_number, /* Number of occ.s of result      O */
     GTRSTATUS    *GTR_status);       /* Status codes                   O */

/*-----                      Search by Query                         -----*/
void GTR_LINK GTRsearchQuery(
     UBYTE1        query[],            /* Query with GTR's format       I */
     BYTE1         index_name[],       /* Index name                    I */
     BYTE1         index_path[],       /* Index directory               I */
     BYTE1         output_fname[],     /* Result file name              I */
     BYTE1         output_path[],      /* Result file derectory         I */
     UBYTE1        codepage,           /* codepage                      I */
                                       /* same as GTRHCREATEPARM          */
     RTRVRANK      *rank_info_ptr,     /* Parameters for ranking        I */
     INT4          discon_count,       /* Maximum occurrences           I */
     INT4          *answer_doc_number, /* Number of documents of result O */
     INT4          *answer_occ_number, /* Number of occ of result       O */
     GTRSTATUS     *GTR_status);       /* Status codes                  O */

/*-----            Search by an array of search terms                -----*/
void GTR_LINK GTRsearch(
     INT4          number_of_words,    /* Number of search terms        I */
     RTRVWORD      search_word[],      /* Search term and parameters    I */
     BYTE1         search_order[],     /* Query with search term No.    I */
                                       /* (Ex.)  1 * ( 2 + 3 )            */
     BYTE1         index_name[],       /* Index name                    I */
     BYTE1         index_directory[],  /* Index directory               I */
     BYTE1         output_filename[],  /* Result filename               I */
     BYTE1         freq_file_name[],   /*                                 */
     BYTE1         output_directory[], /* Result file directory         I */
     UBYTE1        codepage,           /* codepage                      I */
                                       /*     same as GTRHCREATEPARM      */
     RTRVRANK      *rank_info_ptr,     /* Paremeters for ranking        I */
     INT4          discon_count,       /* Maximum occurrences           I */
     void          *reserved_ptr,      /* reserved                        */
     INT4          *ansdocnum,         /* Number of documents of result O */
     INT4          *ansoccnum,         /* Number of occ of result       O */
     GTRSTATUS     *gsp);              /* Status codes                  O */

/*-----                        Item  Search                          -----*/
void GTR_LINK GTRitemSearch(
     INT4           number_of_item_data,  /* Number of items            I */
     SEARCHITEM     item_data[],          /* Item data                  I */
     BYTE1          search_order[],       /* Query with search term No. I */
     INT4           reserved[],           /*  (reserved)                  */
     BYTE1          index_name[],         /* Index name                 I */
     BYTE1          index_directory[],    /* Index directory            I */
     BYTE1          output_filename[],    /* Result filename            I */
     BYTE1          output_directory[],   /* Result file directory      I */
     UBYTE1         codepage,             /* codeapage                  I */
     INT4           discon_count,         /* Maximum occ / doc          I */
     INT4           *answer_number_of_documents, /* Number of doc       O */
     GTRSTATUS      *GTR_status           /* Status code                O */
     );

/*------------------------------------------------------------------------*/
/*   3. Get Hit Position for Highlightingh                                */
/*------------------------------------------------------------------------*/
/*                                                                        */
/*   Calling sequence :                                                   */
/*                                                                        */
/*        GTRgetHitinfo(  .. , &GTR_status);                              */
/*        if (GTR_status.retcode != 0) goto laberr;                       */
/*                                                                        */
/*      laberr:                                                           */
/*        GTRgetMessage(&GTR_status, .. );                                */
/*                                                                        */
/*------------------------------------------------------------------------*/
void GTR_LINK GTRgetHitinfo(
     const UBYTE1   *document_ptr,    /* Ptr. to document contents      I */
     INT4            document_length, /* Length of document contents    I */
     INT4            number_of_occs,  /* Number of occurrences in occs  I */
     OCC             occs[],          /* Occ (fetch from result file)   I */
     INT4            start_position,  /* Position No. of top char       I */
     HITOFFSET       hit_info[],      /* Offset pairs of hit part       O */
     UBYTE1          codepage,        /* Codepage                       I */
                                      /* same as GTRHCREATEPARM           */
     GTRSTATUS       *GTR_status);    /* Status codes                   O */


  /*-----------------------------------------------------------------------*/
  /*   4. Merge the main index and supplemental index                      */
  /*-----------------------------------------------------------------------*/
  /*   Calling sequence :                                                  */
  /*                                                     ---               */
  /*        GTRmergeIndex( .. , &GTR_status);             |                */
  /*        if (GTR_status.retcode != 0) goto laberr;     |                */
  /*                                                      |  ---           */
  /*        GTRmergeIndexFinish( .. , &GTR_status);       |   |            */
  /*        if (GTR_status.retcode != 0) goto laberr;     |   |            */
  /*                                                     --- ---           */
  /*      laberr:                                       CR,E SE,E          */
  /*         GTRgetMessage(&GTR_status, .. );                              */
  /*                                                                       */
  /*-----------------------------------------------------------------------*/
  /*--- Merge the main and supplemental into the temporary index        ---*/
  #define GTRmergeIndex GTR_mergeIndex
  void GTR_LINK  GTRmergeIndex(
        void **work_area_ptr,               /*                           O */
        const BYTE1 index_name[], /*Index name to be merged I */ /* @4-220A */
        const BYTE1 index_directory[],/* Index directory        I */ /* @4-220A */
        const BYTE1 work_dir[],    /* Work file directory       I */ /* @4-220A */
        /* const BYTE1 index_name[IDXNAMELENMAX],  */ /*Index name to be merged I */ /* @4-220D */
        /* const BYTE1 index_directory[DIRLENMAX], */ /* Index directory        I */ /* @4-220D */
        /* const BYTE1 work_dir[DIRLENMAX],     */ /* Work file directory       I */ /* @4-220D */
        GTRSYSPARM  *system_parameter_ptr,  /* System parameters         I */
        GTRSTATUS   *GTR_status             /* Status codes              O */
        );

  /*---   Completion of merging  -  Rename temporary to target          ---*/
  #define GTRmergeIndexFinish GTR_mergeIndexFinish
  void GTR_LINK  GTRmergeIndexFinish(
        void      *work_area_ptr,          /* Ptr. set by GTRmergeIndex  I */
        BYTE1      go_or_cancel,           /* GO_MERGE  | CANCEL_MERGE   I */
                   #define GO_MERGE      GO_CREATE
                   #define CANCEL_MERGE  CANCEL_CREATE
        GTRSTATUS *GTR_status              /*                        O */
                                           /* status               @3-023  */
        );


  /*-----------------------------------------------------------------------*/
  /*   5. Delete information on specified documents from GTR index         */
  /*-----------------------------------------------------------------------*/
  /*   Calling sequence :                                                  */
  /*                                                    ---                */
  /*        GTRdeleteDoc( .. , &GTR_status);             |                 */
  /*        if (GTR_status.retcode != 0) goto laberr1;   |                 */
  /*                                                     |  ---            */
  /*        GTRdeleteDocFinish( .. , &GTR_status);       |   |             */
  /*        if (GTR_status.retcode != 0) goto laberr1;   |   |             */
  /*                                                    --- ---            */
  /*      laberr:                                      CR,E RE,E           */
  /*         GTRgetMessage(&GTR_status, .. );                              */
  /*                                                                       */
  /*-----------------------------------------------------------------------*/
  /*-----         Delete information on specified documents           -----*/
  /* #define GTRdeleteDoc GTR_deleteDoc                            @4-002D */
  void GTR_LINK  GTRdeleteDoc(
        void **work_area_ptr,                 /*                         O */
/*         const BYTE1 index_name[IDXNAMELENMAX],  */  /* Index name            I */ /* @4-220D */
/*         const BYTE1 in_out_directory[DIRLENMAX], */ /* Index directory       I */ /* @4-220D */
/*         const BYTE1 work_directory[DIRLENMAX],   */ /* Work file directory   I */ /* @4-220D */
        const BYTE1 index_name[],             /* Index name            I */ /* @4-220A */
        const BYTE1 in_out_directory[],       /* Index directory       I */ /* @4-220A */
        const BYTE1 work_directory[],         /* Work file directory   I */ /* @4-220A */
        INT4  delete_doc_NO[],                /* Document Numbers        I */
        INT4  number_of_delete_doc,           /* Number of doc No.s      I */
        GTRSYSPARM *system_parameter_ptr,     /* System parameters       I */
        GTRSTATUS *GTR_status                 /* Status codes            O */
        );

  /*-----                        Completion                           -----*/
  /* #define GTRdeleteDocFinish GTR_deleteDocFinish                @4-002D */
  void GTR_LINK  GTRdeleteDocFinish(
        void *work_area_ptr,                /* Ptr. set by GTRdeleteDoc  I */
        BYTE1      go_or_cancel,            /* GO_DELETE | CANCEL_DELETE I */
                   #define GO_DELETE      GO_CREATE
                   #define CANCEL_DELETE  CANCEL_CREATE
        GTRSTATUS *GTR_status               /* Status codes              O */
        );

  /*-----------------------------------------------------------------------*/
  /*   6. Delete Index                                                     */
  /*-----------------------------------------------------------------------*/
  /*   Calling sequence :                                                  */
  /*                                                    --- ---            */
  /*        GTRdeleteIndex( .. , &GTR_status);           |   |             */
  /*        if (GTR_status.retcode != 0) goto laberr1;   |   |             */
  /*                                                    --- ---            */
  /*      laberr:                                      CR,E SE,E           */
  /*         GTRgetMessage(&GTR_status, .. );                              */
  /*                                                                       */
  /*-----------------------------------------------------------------------*/
  #define GTRdeleteIndex GTR_deleteIndex
  void GTR_LINK  GTRdeleteIndex(
        BYTE1 function,                     /*                           I */
                        /*---  Delete only the supplemental index       ---*/
                        /*---  of both Text Index and Item Index        ---*/
                        #define DELETE_INDEX_DELTA     0x04     /* @4-013A */

                        /*---  Delete both main and supplemental        ---*/
                        /*---  of both Text Index and Item Index        ---*/
                        #define DELETE_INDEX_ALL       0x03     /* @4-013A */

                        /*---  Delete both main and supplemental        ---*/
                        /*---  of Text Index only                       ---*/
                        #define DELETE_INDEX_TEXT_ONLY 0x01     /* @4-013A */

                        /*---  Delete both main and supplemental        ---*/
                        /*---  of Item Index only                       ---*/
                        #define DELETE_INDEX_ITEM_ONLY 0x02     /* @4-013A */

                        /* #define DELETE_INDEX_DELTA     'D' */ /* @4-013D */
                        /* #define DELETE_INDEX_ALL       'A' */ /* @4-013D */
                        /* #define DELETE_INDEX_TEXT_ONLY 'T' */ /* @4-013D */
                        /* #define DELETE_INDEX_ITEM_ONLY 'I' */ /* @4-013D */

        const BYTE1 index_name[],               /* Index name             I */ /* @4-220A */
        const BYTE1 index_directory[],          /* Index directory        I */ /* @4-220A */
        /* const BYTE1 index_name[IDXNAMELENMAX],   */ /* Index name             I */ /* @4-220D */
        /* const BYTE1 index_directory[DIRLENMAX],  */ /* Index directory        I */ /* @4-220D */
        GTRSTATUS *GTR_status                   /* Status codes           O */
        );


  /*-----------------------------------------------------------------------*/
  /*   7. Copy / Move Index, Rename Index                                  */
  /*-----------------------------------------------------------------------*/
  /*   Calling sequence :                                                  */
  /*                                                    --- ---            */
  /*        GTRmoveIndex(.. , &GTR_status);              |   |             */
  /*        if (GTR_status.retcode != 0) goto laberr1;   |   |             */
  /*                                                    --- ---            */
  /*      laberr:                                      CR,E SE,E           */
  /*         GTRgetMessage(&GTR_status, .. );                              */
  /*                                                                       */
  /*-----------------------------------------------------------------------*/
  #define GTRmoveIndex GTR_moveIndex
  void GTR_LINK  GTRmoveIndex(
        BYTE1 function,                       /* MOVE_INDEX | COPY_INDEX I */
                      /*--  Move  --*/
                      #define MOVE_INDEX    'M'
                      /*--  Copy  --*/
                      #define COPY_INDEX    'C'
                      /*--  Rename  --*/
                      #define RENAME_INDEX  'R'

        const BYTE1 index_name[],               /* Index name            I */ /* @4-220A */
        const BYTE1 index_directory[],          /* Index directory       I */ /* @4-220A */
        /* const BYTE1 index_name[IDXNAMELENMAX],   */ /* Index name            I */ /* @4-220D */
        /* const BYTE1 index_directory[DIRLENMAX],  */ /* Index directory       I */ /* @4-220D */
        const BYTE1 target_dir_or_target_name[],/* Target directory      I */
                   /* Target directory name when function = MOVE_INDEX   I */
                   /*                            function = COPY_INDEX     */
                   /* Target index name     when function = RENAME_INDEX   */
        GTRSTATUS *GTR_status                 /* Status codes            O */
        );

  /*-----------------------------------------------------------------------*/
  /*   8. Verify index                                                     */
  /*-----------------------------------------------------------------------*/
  /*   Calling sequence :                                                  */
  /*                                              ---                      */
  /*        GTRverifyIndex( ... , &GTR_status);    |                       */
  /*                                              ---                      */
  /*        if (GTR_status.retcode != 0)          SE,S                     */
  /*          GTRgetMessage(&GTR_status, .. );                             */
  /*                                                                       */
  /*-----------------------------------------------------------------------*/
  void GTR_LINK  GTRverifyIndex(
        /* const BYTE1 index_name[IDXNAMELENMAX],    */ /* Index name            I */ /* @4-220D */
        /* const BYTE1 index_directory[DIRLENMAX],   */ /* Index directory       I */ /* @4-220D */
        /* const BYTE1 outputfile_name[FNAMELENMAX], */ /* Output fname (full path) I */ /* @4-220D */
        const BYTE1 index_name[],              /* Index name            I */ /* @4-220A */
        const BYTE1 index_directory[],         /* Index directory       I */ /* @4-220A */
        const BYTE1 outputfile_name[],         /* Output fname (full path) I */ /* @4-220A */
        BYTE1 flag,
             #define   VERIFY_TEXT_INDEX       0x40
             #define   VERIFY_ITEM_INDEX       0x00
             #define   VERIFY_DISPLAY_LOG      0x00
             #define   VERIFY_NOT_DISPLAY_LOG  0x08
        INDEXDOCINFO *document_info_ptr,    /* Information              O */
        GTRSTATUS    *GTR_status            /* Status codes             O */
        );

  /*---   In the output file, document numbers in the index are        ---*/
  /*---   written. Each doc No. is long integer (4-bytes).             ---*/


  /*-----------------------------------------------------------------------*/
  /*   9. Get Index Information                                            */
  /*-----------------------------------------------------------------------*/
  /*   Calling sequence :                                                  */
  /*                                                       ---             */
  /*        GTRgetIndexInfo( .. , &GTR_status);             |              */
  /*                                                       ---             */
  /*        if (GTR_status.retcode != 0)                   SE,S            */
  /*          GTRgetMessage(&GTR_status, .. );                             */
  /*                                                                       */
  /*-----------------------------------------------------------------------*/
  INT4 GTR_LINK  GTRgetIndexInfo(             /* return Max. doc number    */
        /* const BYTE1 index_name[IDXNAMELENMAX],  */ /* Index name             I */ /* @4-220D */
        /* const BYTE1 index_directory[DIRLENMAX], */ /* Index directory        I */ /* @4-220D */
        const BYTE1 index_name[],              /* Index name             I */ /* @4-220A */
        const BYTE1 index_directory[],         /* Index directory        I */ /* @4-220A */
        GTRHCREATEPARM *create_parameter_ptr, /* Creation parameters     O */
        INT4 number_of_documents[2],          /* Number of documents     O */
                                              /*   [0] : In main index     */
                                              /*   [1] : In supplemental   */
/*      void *reserved_pointer,                  reserved                  */
        GTRGETINFO *parameter,                /*                         I */
        GTRSTATUS *GTR_status                 /* Status codes            O */
        );

  /*----------------------------------------------------------------------*/
  /*  10. Get Message                                                     */
  /*----------------------------------------------------------------------*/
  /*   Calling sequence :                                                 */
  /*                                                                      */
  /*        GTRgetMessage( .., &GTR_status, .. );                         */
  /*                                                                      */
  /*----------------------------------------------------------------------*/
  void GTR_LINK  GTRgetMessage(
        UBYTE1        codepage,                     /* Codepage same as I */
                                                    /* GTRHCREATEPARM     */
        GTRSTATUS     *GTR_status,                  /* Status codes     I */
        BYTE1         error_message[MESSAGELENMAX]  /* Message text     O */
        );

  /*----------------------------------------------------------------------*/
  /* #define GTRtrace GTR_trace                                           */
  void GTR_LINK GTRtrace(FILE  *trace_file_ptr,     /* Trace output file ptr       */
                const BYTE1 midashi[],         /*                         */
                const BYTE1 variable_name[],   /*                         */
                const void  *ptr_to_variable,   /*                        */
                INT4   length_of_variable  /*                             */
                );


/*-------------------------------------------------------------------------*/
  void GTR_LINK GTRnormalizeWord(
                UBYTE1, const UBYTE1 *, INT4, UBYTE1 *, INT4, INT4 *, UBYTE1,
                GTRSTATUS *);
/*-------------------------------------------------------------------------*/
  void GTR_LINK GTRunnormalizeWord(
                UBYTE1, const UBYTE1 *, INT4, UBYTE1 *, INT4, INT4 *, UBYTE1,
                GTRSTATUS *);

void GTR_LINK GTRconvertSearchCond(
              GTRCONVERT_HANDLE *, GTRSEARCH_COND,
              GTRSEARCH_COND *, GTRSTATUS *GTR_status);

void GTR_LINK GTRconvertSearchCond_Mix(
              GTRCONVERT_HANDLE *, GTRSEARCH_COND,
              GTRSEARCH_COND *, GTRSTATUS *GTR_status);           /* @4-099A */

void GTR_LINK GTRconvertSearchCond_Release(GTRCONVERT_HANDLE *);

/*--------------------   Reserved and Not Supported yet  ------------------*/
  typedef struct {
    INT2                fieldNO;
    BYTE1               rsv0[2];
    UBYTE1              *item_ptr;
    INT4                item_len;
    BYTE1               rsv1[20];
  } REGISTFIELD;

  typedef struct {
    INT2  fieldNO;
    BYTE1 rsv0[2];
    INT4  data_value;
    BYTE1 rsv1[16];
  } REGISTNUMERIC;

  void GTR_LINK GTRcreateFieldIndexOneDoc(void *, INT4, INT4, REGISTFIELD *, GTRSTATUS *);
  void GTR_LINK GTRcreateNumIndexOneDoc(void *, INT4, INT4, REGISTNUMERIC *, GTRSTATUS *);
  void GTR_LINK GTRnumericRetrieve(INT4, RTRVNUMERIC *, BYTE1 *, INT4 *, BYTE1 *, BYTE1 *,
                          BYTE1 *, BYTE1 *, INT4, INT4 *, GTRSTATUS *);
  void GTR_LINK GTRfieldRetrieve(INT4, RTRVFIELD *, BYTE1 *, INT4 *, BYTE1 *, BYTE1 *,
                        BYTE1 *, BYTE1 *, UBYTE1, INT4, INT4 *, GTRSTATUS *);
  void GTR_LINK GTRgetImportant(BYTE1 *, BYTE1 *, UBYTE1 *, INT4, UBYTE1,
                       GTRSTATUS *);
  void GTR_LINK GTRretrieveByDoc(UBYTE1 *, INT4, BYTE1 *, BYTE1 *, BYTE1 *, BYTE1 *,
                        UBYTE1, RTRVRANK *, INT4, INT4 *, INT4 *, GTRSTATUS *);
/*-------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------*/
/*---------------------  For Compatibility with V1 ------------------------*/
#ifdef GTR_V1_MIG
  #define GTRsimpleRetrieve(a,b,c,d,e,f,g,h,i,j)        \
          GTRsimpleSearch(a,b,c,d,e,f,g,NULL,h,i,j)
  #define GTRretrieveQuery(a,b,c,d,e,f,g,h,i,j)         \
          GTRsearchQuery(a,b,c,d,e,f,NULL,g,h,i,j)
  void GTR_LINK GTRretrieve(INT4, RTRVWORD *, RTRVOP, BYTE1, BYTE1 *, BYTE1 *,
                   BYTE1 *, BYTE1 *, UBYTE1, INT4 *, INT4 *, GTRSTATUS *);
  #define GTRgetIndexBasicInfo(a,b,c,d)         \
          GTRgetIndexInfo(a,b,c,NULL,NULL,d)
  #define GTRgetMaxDoc(a,b,c)                   \
          GTRgetIndexInfo(a,b,NULL,NULL,NULL,c)
  #define GTRgetErrorMsg(x,y)         GTRgetMessage(CODEPAGE_SJIS, x, y)
  #define GTRgetErrorMsg_English(x,y) GTRgetMessage(CODEPAGE_I850, x, y)
#endif
/*-------------------------------------------------------------------------*/
void GTR_LINK GTRgetHitOffset(
     const UBYTE1    *document_ptr,   /* Ptr. to document contents      I */
     INT4            document_length, /* Length of document contents    I */
     INT4            number_of_occs,  /* Number of occurrences in occs  I */
     OCC             occs[],          /* Occ (fetch from result file)   I */
     UBYTE1          codepage,        /* Codepage                       I */
                                      /* same as GTRHCREATEPARM           */
     INT4            *start_position, /* Position No. of top char       I */
     HITOFFSET       hit_info[],      /* Offset pairs of hit part       O */
     INT4            *hitinfo_startNo,/*                                O */
     INT4            *hitinfo_endNo,  /*                                O */
     GTRSTATUS       *GTR_status);    /* Status codes                   O */

/*------------------------------------------------------@3-029 BEGIN -----*/
/*   11. Create index / Add to index with Auto-merge                      */
/*------------------------------------------------------------------------*/
/*   Calling sequence :                                                   */
/*                                                                        */
/*        ---  Initialization  ---                           ---          */
/*        GTRcreateIndexInit( .. , &GTR_status);              |           */
/*        if (GTR_status.retcode != 0) goto laberr;           |           */
/*                                                            |           */
/*        for (DocNO = FirstDocNO; Until last doc; ++DocNO) { |           */
/*                                                            |           */
/*          Read one document on memory                       |           */
/*                                                            |           */
/*          --- Process of one document  ---                  |           */
/*          GTRcreateIndexOneDoc( .. , &GTR_status);          |           */
/*                         or                                 |           */
/*          GTRcreateItemIndexOneDoc( .. , &GTR_status);      |           */
/*          if (GTR_status.retcode != 0) goto laberr;         |           */
/*                                                            |           */
/*        }                                                   |           */
/*        --- Create & Merge index with temporary filename ---|           */
/*        GTRcreateIndexLastMerge( .. , &GTR_status);         |           */
/*        if (GTR_status.retcode != 0) goto laberr;           |           */
/*                                                            |           */
/*        --- Completion of index creation and merge ---      |  ---      */
/*        GTRcreateIndexTermMerge( .. , &GTR_status);         |   |       */
/*        if (GTR_status.retcode != 0) goto laberr;           |   |       */
/*                                                           --- ---      */
/*      laberr:                                             CR,E SE,E     */
/*        GTRgetMessage(&GTR_status, .. );                                */
/*                                                                        */
/*------------------------------------------------------------------------*/
  /*----- Create the target index with temporary filenames, and merge ----*/
  void GTR_LINK  GTRcreateIndexLastMerge(
      void *work_area_ptr1,          /* Ptr. set by GTRcreateIndexInit I */
      void **work_area_ptr2,         /* Ptr. set by this API           O */
      int  mergeON,                  /* AutoMerge ON/OFF               I */
          #define AUTOMERGE_ON      1
          #define AUTOMERGE_OFF     0
      GTRSTATUS *GTR_status          /* Status codes                   O */
      );

  /*---   Completion of indexing and merge - Rename temporary to target ---*/
  void GTR_LINK  GTRcreateIndexTermMerge(
      void *work_area_ptr1,          /* Ptr. set by GTRcreateIndexInit I */
      void *work_area_ptr2,          /* Ptr. set by GTRcreateIndexLastMerge I */
      BYTE1 go_or_cancel,            /* GO_CREATEMERGE  | CANCEL_CREATEMERGE I */
          #define GO_CREATEMERGE      GO_CREATE
          #define CANCEL_CREATEMERGE  CANCEL_CREATE
      GTRSTATUS *GTR_status          /* Status codes                   O */
      );
/*------------------------------------------------------@3-029 END -------*/


/*------------------------------------------------------------------------*/
/*   12. Create index of multiple documents                               */
/*------------------------------------------------------------------------*/
/*   Calling sequence :  -- Create Text-Index of mutiple documents--      */
/*                                                                        */
/* #include <gtr.h>                                                       */
/* #include <gtrtype.h>                                                   */
/* ...                                                                    */
/*                                                                        */
/*    GTRDOCINFO      gtrdocinfo_array[MAX_NUM_DOCS];                     */
/*    long            num_docs;                                           */
/*    INT2*           delimit;                                            */
/*    UBYTE1**        delimit_ptr;                                        */
/*    FIELDBOUNDARY*  fieldboundary;                                      */
/*    BOUNDARYINFO    boundaryinfo;                                       */
/*                                                                        */
/*        ---  Initialization  ---                                        */
/*        GTRcreateIndexInit( .. , &GTR_status);                          */
/*        if (GTR_status.retcode != 0) goto laberr;                       */
/*                                                                        */
/*    //  Prepare data blocks prior to API call                           */
/*    for (ldoc = 0;ldoc < num_docs ; ldoc++) {                           */
/*                                                                        */
/*       gtrdocinfo_array[ldoc].docno = start_docno + ldoc;               */
/*                                                                        */
/*       ------------------------------------                             */
/*         Here, prepare the text in strbuff.                             */
/*       ------------------------------------                             */
/*                                                                        */
/*       gtrdocinfo_array[ldoc].docptr = strbuff;                         */
/*       gtrdocinfo_array[ldoc].doclen = strlen(strbuff);                 */
/*                                                                        */
/*       memset(&(gtrdocinfo_array[ldoc].boundaryinfo), 0x00,..// reset   */
/*                                                                        */
/*       // Information used for Adjacent Search                          */
/*       gtrdocinfo_array[ldoc].boundaryinfo.boundary_num = 2;            */
/*       delimit = malloc(...                                             */
/*       gtrdocinfo_array[ldoc].boundaryinfo.delimiter_NO = delimit;      */
/*       delimit[0] = 1;   // Separating rule number                      */
/*       delimit[1] = 2;                                                  */
/*       delimit_ptr = malloc(...                                         */
/*       gtrdocinfo_array[ldoc].boundaryinfo.boundary_ptr = delimit_ptr;  */
/*       delimit_ptr[0] = (UBYTE1*)strbuff;                               */
/*       delimit_ptr[1] = ...;                                            */
/*                                                                        */
/*       // Information used for Field Search                             */
/*       gtrdocinfo_array[ldoc].boundaryinfo.fldboundary_num = 3;         */
/*       fieldboundary = malloc(...                                       */
/*       gtrdocinfo_array[ldoc].boundaryinfo.fldboundary = fieldboundary; */
/*                                                                        */
/*       memset(fieldboundary, 0x00, sizeof(fieldboundary));              */
/*       fieldboundary[0].fld_number = 3;      // Field number            */
/*       fieldboundary[0].fld_start_ptr= ...                              */
/*       fieldboundary[0].fld_end_ptr  = ...                              */
/*                                                                        */
/*       fieldboundary[1].fld_number = 11;     // Field number            */
/*       fieldboundary[1].fld_start_ptr= ...                              */
/*       fieldboundary[1].fld_end_ptr  = ...                              */
/*                                                                        */
/*       fieldboundary[2].fld_number = 15;     // Field number            */
/*       fieldboundary[2].fld_start_ptr= ...                              */
/*       fieldboundary[2].fld_end_ptr  = ...                              */
/*       .................................................                */
/*    }                                                                   */
/*                                                                        */
/*                                                                        */
/*    lrc =  GTRcreateIndexPluralDoc(                                     */
/*                   cwp,                 // index handle                 */
/*                   num_docs,            // number of documents          */
/*                   gtrdocinfo_array,    // data blocksv                 */
/*                   &num_processed,                                      */
/*                   &gtrstat                                             */
/*           );                                                           */
/*                                                                        */
/*                                                                        */
/*    if (lrc != GTR_PLURALNORMALEND || num_processed != num_docs) {      */
/*       printf("\nError!! GTRcreateIndexPluralDoc");                     */
/*       if(lrc == GTR_PLURALERROR_ONE) {                                 */
/*          GTRgetMessage(codepage, &gtrstat, gtrerrmsg);                 */
/*          printf("\n::at %d-th block::%s",num_processed, gtrerrmsg);    */
/*       }                                                                */
/*       else if(lrc == GTR_PLURALERROR_INVALIDPARA)                      */
/*          printf("\n::parameter error");                                */
/*       goto laberr1;                                                    */
/*    }                                                                   */
/*                                                                        */
/*    for (ldoc = 0;ldoc < num_docs ; ldoc++) {                           */
/*        // release allocated memory                                     */
/*    }                                                                   */
/*                                                                        */
/* laberr:                                                                */
/*  GTRgetMessage(&GTR_status, .. );                                      */
/*                                                                        */
/*------------------------------------------------------------------------*/
/*  Calling sequence :  -- Create Item-Index of mutiple documents--       */
/* #include <gtr.h>                                                       */
/* #include <gtrtype.h>                                                   */
/*     ....                                                               */
/*                                                                        */
/*  GTRITEMINFO     gtriteminfo_array[MAX_NUM_DOCS];                      */
/*  REGISTITEM      registitem[MAX_NUM_ITEMS];                            */
/*                                                                        */
/*        ---  Initialization  ---                                        */
/*        GTRcreateIndexInit( .. , &GTR_status);                          */
/*        if (GTR_status.retcode != 0) goto laberr;                       */
/*                                                                        */
/*  for (ldoc = 0;ldoc < num_docs ; ldoco++) {                            */
/*                                                                        */
/*     gtriteminfo_array[ldoc].docno = start_docno + ldoc;                */
/*                                                                        */
/*     gtriteminfo_array[ldoc].registitem_num = 4;                        */
/*                                                                        */
/*     registitem = (REGISTITEM*) malloc(...                              */
/*                                                                        */
/*     memset(registitem, 0x00, sizeof(registitem));                      */
/*                                                                        */
/*     gtriteminfo_array[ldoc].registitem = registitem;                   */
/*                                                                        */
/*     registitem[0].item_number = 11;               //  item number      */
/*     registitem[0].item_type   = ITEMTYPE_CHAR;    //  character type   */
/*     registitem[0].char_ptr    = ...               //                   */
/*     registitem[0].char_len    = ...               //                   */
/*                                                                        */
/*     registitem[1].item_number = 21;               //  item number      */
/*                                                                        */
/*     ...                                                                */
/*                                                                        */
/*  }                                                                     */
/*                                                                        */
/*  lrc = GTRcreateItemIndexPluralDoc(                                    */
/*          cwp2,               // index handle                           */
/*          num_docs,           // number of documents                    */
/*          gtriteminfo_array,                                            */
/*          &num_processed,                                               */
/*          &gtrstat                                                      */
/*        );                                                              */
/*                                                                        */
/*                                                                        */
/*    if (lrc != GTR_PLURALNORMALEND || num_processed != num_docs) {      */
/*       printf("\nError!! GTRcreateItemIndexPluralDoc");                 */
/*       if(lrc == GTR_PLURALERROR_ONE) {                                 */
/*          GTRgetMessage(codepage, &gtrstat, gtrerrmsg);                 */
/*          printf("\n::at %d-th block::%s",num_processed, gtrerrmsg);    */
/*       }                                                                */
/*       else if(lrc == GTR_PLURALERROR_INVALIDPARA)                      */
/*          printf("\n::parameter error");                                */
/*       goto laberr1;                                                    */
/*    }                                                                   */
/*                                                                        */
/*    for (ldoc = 0;ldoc < num_docs ; ldoc++) {                           */
/*        // release allocated memory                                     */
/*    }                                                                   */
/*                                                                        */
/* laberr:                                                                */
/*  GTRgetMessage(&GTR_status, .. );                                      */
/*                                                                        */
/*------------------------------------------------------------------------*/
  /*----- Create Text-Index of mutiple documents--------------------------*/
long GTR_LINK  GTRcreateIndexPluralDoc(
      void*             cwp,               /* Handle of Text-Index        */
      INT4              num_docs,          /* size of gtrdocinfo_array    */
      GTRDOCINFO*       gtrdocinfo_array,  /* document info array         */
      INT4*             num_processed,     /* num of processed docs       */
      GTRSTATUS*        gsp
);
  /*----- Create Item-Index of mutiple documents--------------------------*/
long GTR_LINK  GTRcreateItemIndexPluralDoc(
      void*             cwp,               /* Handle of Item-Index        */
      INT4              num_docs,          /* size of gtriteminfo_array   */
      GTRITEMINFO*      gtriteminfo_array, /* item document info array    */
      INT4*             num_processed,     /* num of processed item docs  */
      GTRSTATUS*        gsp
   ) ;
/*------------------------------------------------------------------------*/
INT4 GTR_LINK GTRStemUtil(                /* @3-073A */
      char*              word1,           /* @3-073A */
      char*              word2,           /* @3-073A */
      INT2               word2len,        /* @3-073A */
      GTRSTATUS*         GTR_status       /* @3-073A */
     );                                   /* @3-073A */
#ifdef GTR_CALLBACKS                      /* @3-073A */
void GTR_LINK GTRSetupCallbackInfo(       /* @3-073A */
      FTGCallbackStream *pMS              /* @3-073A */
     );                                   /* @3-073A */
#endif                                    /* @3-073A */
/*--- @4-003 add start ---*/
void GTR_LINK GTRgetKeyInfo(UBYTE1     index_name[],
                            UBYTE1     index_dir[],
                            GTRKEYINFO *pGtrKeyInfo,
                            UINT4      number_of_documents[2],
                            GTRSTATUS  *pGtrStatus);
/*--- @4-003 add end ---*/
/*--- @4-004 add start ---*/
void   GTR_LINK GTRkeyIteratorInit(UBYTE1         index_name[],
                                       UBYTE1         index_dir[],
                                   UBYTE1         index_purpose,   /* @4-011A */
                                   GTRKEYITERATOR *pKeyIterator,
                                   GTRSTATUS      *gsp);
void   GTR_LINK GTRkeyIteratorTerm(GTRKEYITERATOR keyiterator,
                                   GTRSTATUS      *gsp);
UBYTE1 GTR_LINK GTRkeyIterator_HasMoreKeys(GTRKEYITERATOR keyiterator,
                                           GTRSTATUS      *gsp);
void   GTR_LINK GTRkeyIterator_GetNextKey(GTRKEYITERATOR keyiterator,
                                          GTRSTATUS      *gsp);
void   GTR_LINK GTRkeyIterator_JumpToKey(GTRKEYITERATOR keyiterator,
                                         UBYTE1         pattern[],
                                         UINT4          pattern_len,
                                         GTRSTATUS      *gsp);
void   GTR_LINK GTRkeyIterator_GetKeyString(GTRKEYITERATOR keyiterator,
                                            UBYTE1         pattern[],
                                            UINT4          pattern_buff_len,
                                            UINT4          *pattern_len,
                                            GTRSTATUS      *gsp);
UINT4  GTR_LINK GTRkeyIterator_GetDocumentCount(GTRKEYITERATOR keyiterator,
                                                GTRSTATUS      *gsp);
UINT4  GTR_LINK GTRkeyIterator_GetHitCount(GTRKEYITERATOR keyiterator,
                                           GTRSTATUS      *gsp);
/*--- @4-004 add end ---*/
/*--- @4-250A start ---*/
void GTR_LINK GTRkeyIterator_SearchKeyString(GTRKEYITERATOR keyiterator,
                                    UBYTE1         pattern[],
                                    UINT4          pattern_len,
                                    GTRSTATUS      *gsp);
/*--- @4-250A end ---*/
/*--- @4-011 add start ---*/
void   GTR_LINK GTRkeyIterator_GetItemString(GTRKEYITERATOR keyiterator,
                                            UBYTE1         pattern[],
                                            UINT4          pattern_buff_len,
                                            UINT4          *pattern_len,
                                            GTRSTATUS      *gsp);
/*--- @4-011 add end ---*/
/*--- @4-052 add start ---*/
void GTR_LINK GTRkeyIterator_GetItem(GTRKEYITERATOR keyiterator,        /* @4-250C */
                       BYTE1          *item_type,
                       UINT2          *item_number,
                       UBYTE1         pattern[],
                       UINT4          pattern_buff_len,
                       UINT4          *pattern_len,
                       FLOAT8         *num_value,
                       GTRSTATUS      *gsp);
/*--- @4-052 add end ---*/
/*--- @4-010 add start ---*/
void   GTR_LINK GTRposIteratorInit(GTRPOSITERATOR *pPosIterator,
                                   GTRKEYITERATOR keyIterator,
                       GTRSTATUS      *gsp);
void   GTR_LINK GTRposIteratorTerm(GTRPOSITERATOR positerator,
                                   GTRSTATUS      *gsp);
UBYTE1 GTR_LINK GTRposIterator_HasMorePoss(GTRPOSITERATOR positerator,
                                           GTRSTATUS      *gsp);
void   GTR_LINK GTRposIterator_GetNextPos(GTRPOSITERATOR positerator,
                                          GTRSTATUS      *gsp);
UINT4  GTR_LINK GTRposIterator_GetDocumentNumber(GTRPOSITERATOR positerator,
                                                 GTRSTATUS      *gsp);
UINT4  GTR_LINK GTRposIterator_GetDocumentPosition(GTRPOSITERATOR positerator,
                                                   GTRSTATUS      *gsp);
/*--- @4-010 add end ---*/

/*--- @4-180A Start ---*/
/*--- GTR initialize call ---*/
void GTR_LINK GTRinit(GTRSTATUS *);

/*--- GTR terminate call ---*/
void GTR_LINK GTRterm(GTRSTATUS *);

/*--- Initialize function for each thread ---*/
void GTR_LINK GTRinitThread(GTRSTATUS *);

/*--- Terminate function for each thread ---*/
void GTR_LINK GTRtermThread(GTRSTATUS *);
/*--- @4-180A End   ---*/


#ifdef  __cplusplus                                     /* @3-043D *//* @4-204A */
/*#if defined __cplusplus && !defined GTR_CPP  */       /* @3-043A *//* @4-204D */
  }
#endif
#endif
