/***************************************************************************/
/*  FILE NAME   : GTRtype.h                                                */
/***************************************************************************/
/* Licensed Materials - Property of IBM                                    */
/* Global Text Retrieval (GTR)                                             */
/* (C) Copyright IBM Corporation 1996, 2001.                               */
/* All rights reserved.                                                    */
/*                                                                         */
/* US Government Users Restricted Rights - Use, duplication or             */
/* disclosure restricted by GSA ADP Schedule Contract with                 */
/* IBM Corporation.                                                        */
/***************************************************************************/
#ifndef GTRtype
#define GTRtype

#ifdef _WIN32
#pragma pack(push, 4)
#endif
/*************************************************************************/
/*                          Type Definition                              */
/*************************************************************************/
  typedef unsigned long       UINT4;
  typedef signed   long       INT4;
  typedef unsigned short      UINT2;
  typedef signed   short      INT2;
  typedef char                BYTE1;
  typedef unsigned char       UBYTE1;

  typedef float               FLOAT4;
  typedef double              FLOAT8;

/*************************************************************************/
/*                             Constants                                 */
/*************************************************************************/
  /*---  Document number Min. ---*/
  #define OKMINDOC   0

  /*---  Document number Max. ---*/
  #define OKMAXDOC   0x7FFFFFFE                               /* @4-041A */
/*  #define OKMAXDOC   268435455 */                           /* @4-041D */

  /*---  Position number Min. ---*/
  #define OKMINPOS   1

  /*---  Position number Max. ---*/
  #define OKMAXPOS   1073741823

  /*---  Block number Min.    ---*/
  #define OKMINBLOCK 1

  /*---  Block number Max.    ---*/
  #define OKMAXBLOCK 0X7FFFFFFE

  /*---  Part number Min.     ---*/
  #define OKMINPART 0

  /*---  Part number Max.     ---*/
  #define OKMAXPART  0x7FFFFFFE

  /*---  Separate-rule No. Min. ---*/
  #define DELIMITERNO_MIN    0
  /*---  Separate-rule No. Max. ---*/
  #define DELIMITERNO_MAX   32
  #define SENTENCE_DLMNO     0
  #define WORD_BOUNDARY     39

  /*---  Max. length of index name (including '\0') ---*/
  /*   #define IDXNAMELENMAX 64 */                              /* @4-220D */

  /*---  Max. length of directory name (including '\0') ---*/
  /* #define DIRLENMAX     128 */                               /* @4-220D */

  /*---  Max. length of file name (including '\0') ---*/
  /*   #define FNAMELENMAX   96 */                              /* @4-220D */

  /*---  Maximum length of result file name including directory name (terminated by '\0') ---*/
#if defined(GTR_MAC)  /* @4-250A */
  /* With GTR 4.1.3.2, this grew dramatically and now won't compile on
  ** the mac in gtr_comm.c - temporarily set back to 512 until
  ** this is rectified.  This code isn't even enabled on the Mac
  ** Due to stack size problems
  */
  #define EXTFNAMELENMAX  (512)                                 /* @4-250A */
#else                                                           /* @4-250A */
  #define EXTFNAMELENMAX  (2048)                                /* @4-220D */
#endif                                                          /* @4-250A */

  /*--- If filename exceeds this value, file name will be abbreviated ---*/
  #define ERRFNAMELENMAX  (512)                                 /* @4-220A */

  /*---  Max. length of message text  ---*/
/*   #define MESSAGELENMAX (FNAMELENMAX * 2 + DIRLENMAX * 2 + 64) */ /* @4-220D */
  #define MESSAGELENMAX (ERRFNAMELENMAX * 2 + 64)               /* @4-220A */

  /*---  Length of Version identifier  ---*/                    /* @4-143D */
  /* #define GTRVERSIONLEN 4 */                                 /* @4-143D */

  /*---  Max. number of documents returned by Ranking search  ---*/
  #define RANKCOUNTMAX  2147483647

  /*---  Default value of search term weight  ---*/
  #define DEFAULT_WEIGHT_VALUE 100

  /*---  Max. CharItem Number  ---*/
  #define CHAR_ITEMNUMBERMAX  254

  /*---  Min. CharItem Number  ---*/
  #define CHAR_ITEMNUMBERMIN    1

  /*---  Max. Numeric Item Number  ---*/
  #define NUM_ITEMNUMBERMAX  32767

  /*---  Min. Numberic Item Number  ---*/
  #define NUM_ITEMNUMBERMIN      1

  #define CHAR_ITEMCHARMAX      128
  #define CHAR_ITEMDATANUMMAX  1024

  #define MASKDISCONCOUNT_MIN  1000

  #define IDXDIR_SEPARATOR  '\t'                            /* @3-025A */
  /* #define IDXDIR_SEPARATOR  0x09 */ /* @CDROM_A */ /* @3-019C*/ /* @3-025D*/

  #define ITEMLENMAX  128

  /*--- Just for compatibility (these parameter does not have any sense ---*/

  #define DIRLENMAX     (1792)                                  /* @4-220A */
  #define IDXNAMELENMAX (2048)                                  /* @4-220A */
  #define FNAMELENMAX   (256)                                   /* @4-220A */

/*************************************************************************/
/*                        Structure Definition                           */
/*************************************************************************/
/*------------------------------------------*/
/*  Status (used by all functions commonly) */
/*------------------------------------------*/
typedef struct {
  INT4  retcode;                          /* Return code                 */
  INT4  errcode;                          /* Error code                  */
  BYTE1 retrieve_status;
           #define RETRIEVE_DISCONTINUED   0x01
           #define RETRIEVE_COMPLETED      0x00
           #define RETRIEVE_STOP_WORD      0x02 /* Detected stop words   */ /* @4-028A */
  UBYTE1        processed_index;          /*                             */
                #define  GTR_TEXT_INDEX  0x80
                #define  GTR_ITEM_INDEX  0x40
  BYTE1 rsv0[2];                          /* reserved            @4-035A */
/*   BYTE1 errfname1[DIRLENMAX+FNAMELENMAX];  *//* When retcode = FILE_...     */ /* @4-220D */
                                          /* Filename that caused error  */
/*   BYTE1 errfname2[DIRLENMAX+FNAMELENMAX];  *//* When rercode = FILE_RENAME_ */ /* @4-220D */
  BYTE1 errfname1[ERRFNAMELENMAX];        /* When retcode = FILE_...     */ /* @4-220A */
                                          /* Filename that caused error  */
  BYTE1 errfname2[ERRFNAMELENMAX];        /* When rercode = FILE_RENAME_ */ /* @4-220A */
                                          /* ERROR, new filename         */
/*BYTE1 reserved[32];                        resrved                     */
/*BYTE1 reserved[31];                        reserved  @3-069D           */
  INT4  system_errno;                     /* errno     @3-069A           */
  /* BYTE1 reserved[24]; */ /* reserved     27->24 @4-035C */ /* @4-038D */
  INT4  errinfo;                          /* error information   @4-038A */
  BYTE1 reserved[20];                     /* reserved            @4-038A */
} GTRSTATUS;


/*------------------------------------------*/
/*         For Index Creation               */
/*------------------------------------------*/
/*---  System parameters   ---*/
typedef struct {
  INT4  poolsize;               /* Primary memory size         w : -1 */
          /* Primary memory size unspecified : -1 @3-023 */
  INT4  poolsizedelta;          /* Secondary memory size       w : -1 */
        /* Secondary memory size unspecified : -1 @3-023 */
  INT4  filemax;                /* Max. number of files opened concurrently*/
  BYTE1 displaylog;             /* Logs are output to stdout or not        */
                     /*--  Output log to stdout            --*/
                     #define DISPLAY_LOG      'D'
                     /*--  Not output log to stdout        --*/
                     #define NOT_DISPLAY_LOG  'N'
  BYTE1 overwrite;             /* overwrite mode  @4-053A */
                     #define GTR_OVERWRITE_ON  'Y'       /* overwrite on  */ /* @4-053A */
                     #define GTR_OVERWRITE_OFF 'N'       /* overwrite off */ /* @4-053A */
  BYTE1 singleworkfile;        /* work file control @4-054A */
                     #define GTR_SINGLEWORKFILE_ON   'Y' /* use single workfile  */ /* @4-054A */
                     #define GTR_SINGLEWORKFILE_OFF  'N' /* use normal workfiles */ /* @4-054A */
  BYTE1 reserved[33];                             /* 35 -> 33 @3-061C @3-062C */
} GTRSYSPARM;

#define OVERWRITE_ON     GTR_OVERWRITE_ON  /* overwrite ON @3-061A */ /* add 'GTR' @4-053C */
#define OVERWRITE_OFF    GTR_OVERWRITE_OFF /* overwrite OFF@3-061A */ /* add 'GTR' @4-053C */
#define SINGLEWORKFILE_ON   GTR_SINGLEWORKFILE_ON  /* use single workfile @3-062A */ /* add 'GTR' @4-053C */
#define SINGLEWORKFILE_OFF  GTR_SINGLEWORKFILE_OFF /* use normal workfiles@3-062A */ /* add 'GTR' @4-053C */

/*--- Deleted word list ----------------------------------- @4-028A Start */
typedef struct {
  BYTE1*  pszStopWordList;                    /* Stop word list file name */
  UBYTE1  rsv[28];                                            /* reserved */
} GTRSTOPWORDLIST;                                  /* remove '_' @4-035C */
/*--- Deleted word list ------------------------------------- @4-028A End */

/*---  Indexing parameters (for high) ---*/
typedef struct {
  UBYTE1        index_type;                 /* Index type                 */
                     #define INDEX_TYPE_1 0x01
                     #define INDEX_TYPE_2 0x02
                     #define INDEX_TYPE_3 0x03

  UBYTE1        codepage;                    /* Codepage                  */
                     #define CODEPAGE_SJIS 1
                     #define CODEPAGE_BIG5 2
                     #define CODEPAGE_KOKR 3
                     #define CODEPAGE_SCHN 4
                     #define CODEPAGE_I850 5
                     #define CODEPAGE_850X 6
                     #define CODEPAGE_88591  7
                     #define CODEPAGE_CP1252 8
                     #define CODEPAGE_SCHN_WIN 9
                     #define CODEPAGE_EBCDIC_KANA_KANJI 10      /* @3-019A */
                     #define CODEPAGE_EBCDIC_LATIN_KANJI 11     /* @3-019A */
                     #define CODEPAGE_EBCDIC_TCHN 12            /* @3-019A */
                     #define CODEPAGE_EBCDIC_KOKR 13            /* @3-019A */
                     #define CODEPAGE_EBCDIC_SCHN 14            /* @3-019A */
                     #define CODEPAGE_EBCDIC_500  15            /* @3-019A */
                     #define CODEPAGE_CP1250 16                 /* @3-024A */
                     #define CODEPAGE_CP1251 17                 /* @3-024A */
                     #define CODEPAGE_CP1253 18                 /* @3-024A */
                     #define CODEPAGE_CP1254 19                 /* @3-024A */
                     #define CODEPAGE_CP1255 20                 /* @3-024A */
                     #define CODEPAGE_CP1256 21                 /* @3-024A */
                     #define CODEPAGE_CP1257 22                 /* @3-024A */
                     #define CODEPAGE_CP1258 23                 /* @3-024A */
                     #define CODEPAGE_CP874 24                  /* @3-024A */
                     #define CODEPAGE_EBCDIC_THAI      25       /* @3-031A */
                     #define CODEPAGE_EBCDIC_LATIN2    26       /* @3-031A */
                     #define CODEPAGE_EBCDIC_CYRILLIC  27       /* @3-031A */
                     #define CODEPAGE_EBCDIC_TURKEY    28       /* @3-031A */
                     #define CODEPAGE_EBCDIC_GREEK     29       /* @3-031A */
                     #define CODEPAGE_EBCDIC_HEBREW    30       /* @3-031A */
                     #define CODEPAGE_EBCDIC_ARABIC    31       /* @3-031A */
                     #define CODEPAGE_EBCDIC_BALTIC    32       /* @3-031A */
                     #define CODEPAGE_EBCDIC_ESTONIA   33       /* @3-031A */
                     #define CODEPAGE_UTF8             34       /* @U-001A */
                     #define CODEPAGE_EUC_JP           35       /* @3-051A */
                     #define CODEPAGE_EUC_TW           36       /* @3-051A */

  UBYTE1        index_purpose;
                     #define TEXT_SEARCH     1          /* @4-002C  0 -> 1 */
                     #define ITEM_SEARCH     2          /* @4-002C  3 -> 2 */
  UBYTE1        disable;
                     #define ENABLE_ALL    0x00
                     #define CASE_DISABLE  0x01
                     #define SAME0_DISABLE 0x02
  INT4          vvglength;                   /* Compression factor       */
/*BYTE1         reserved[32];                                            */
  UBYTE1        word_rule;                                    /* @WORD_A */
                     #define WORDRULE_ALPHA    0              /* @WORD_A */
                     #define WORDRULE_ALPHANUM 1              /* @WORD_A */
/*BYTE1         reserved[31];                                    @XGRM_D */
  UBYTE1        xindex_type;                                  /* @XGRM_A */
                     #define XINDEX_TYPE_1 0x01
                     #define XINDEX_TYPE_2 0x02
                     #define XINDEX_TYPE_3 0x03
/*BYTE1         reserved[30];                      @4-026D */ /* @XGRM_A */
  /*--- @4-026A Start ---------------------------------------------------*/
  INT2          block_threshold;  /* Starting size for blocked structure */
  INT2          block_size;              /* Block size for position data */
  /* BYTE1         reserved[26]; */                           /* @4-028D */
  /*--- @4-026A End   ---------------------------------------------------*/
  UBYTE1        reserved2;                                    /* @4-028A */
  /*--- @4-028A Start ---------------------------------------------------*/
  UBYTE1        stop_word_list_num;          /* Number of stop word list */
  GTRSTOPWORDLIST  *paStopWordList;      /* Array of stop word's list */
  BYTE1         reserved[20];
  /*--- @4-028A End   ---------------------------------------------------*/
} GTRHCREATEPARM;

  #define CODEPAGE_TEXT_MIN CODEPAGE_SJIS
  #define CODEPAGE_TEXT_MAX CODEPAGE_EUC_TW /* @3-019C @3-024C @3-031C @U-001C @3-051C */


/*-- Field boundary information (for high) (Start of part added by @FLD)--*/
typedef struct {
  UINT2          fld_number;        /* Field number                       */
  BYTE1          rsv1[2];           /* (reserved)                         */
  UBYTE1         *fld_start_ptr;    /* Pointer to start of field          */
  UBYTE1         *fld_end_ptr;      /* Pointer to end of field            */
  BYTE1          rsv2[20];          /* (reserved)                         */
} FIELDBOUNDARY;
/*---------------------  (End of part added by @FLD)  --------------------*/

/*--  Boundary information (for high)  --*/
typedef struct {
  INT4          boundary_num;   /* Number of boundaries                */
  INT2          *delimiter_NO;  /* Ptr.to Searating-rule No.s array    */
                                /* Number of items is boundary_num.    */
  UBYTE1        **boundary_ptr; /* Ptr.to boundary array. Number of    */
                                /* items is boundary_num. Ptr.s must   */
                                /* be in ascending order.              */
  BYTE1         word_boundary;  /* for Hangul only                     */
          #define  SPECIFIED_BY_UPGM  0x01
          #define  USE_DEFAULT        0x00
/*BYTE1         reserved[31];                                      @FLD_D */
  BYTE1         reserved[3];                                    /* @FLD_A */

  /*-------  for Field Search  (Start of part added by @FLD )  -----------*/
  INT4          fld_boundary_num; /* Number of Field Boundary information */
  FIELDBOUNDARY *fld_boundary;    /* Filed Boundary Information           */

  BYTE1         reserved1[20];    /* (reserved)                           */
  /*---------------------  (End of part added by @FLD)  ------------------*/

} BOUNDARYINFO;

typedef struct {                                              /* @4-002A */
  UINT2   usDocType;                                          /* @4-002A */
          #define GTR_DOC_NEW     0x1                         /* @4-002A */
          #define GTR_DOC_UPDATE  0x2                         /* @4-002A */
  BYTE1   rsv0[30];                                           /* @4-035A */
} GTRONEDOCINFO;                                              /* @4-002A */


/*---  For Item Index Creation  ---*/
  typedef struct {
    INT2           item_number;
    BYTE1          item_type;
                   #define ITEMTYPE_CHAR    'C'
                   #define ITEMTYPE_NUM     'N'
    BYTE1          rsv0[17];

    /*-----  Character Item  -----*/
    UBYTE1         *char_ptr;
    INT2           char_len;
    BYTE1          rsv1[22];

    /*-----  Numeric Item  -----*/
    FLOAT8         num_value;
    BYTE1          rsv2[16];
  } REGISTITEM;


/*------------------------------------------*/
/*              For Search                  */
/*------------------------------------------*/
  /*---                            Range                                ---*/
  typedef struct {
    INT4          start;                     /* Block Number (from)        */
    INT4          end;                       /* Block Number (To)          */
  } RTRVRANGE;


  /*---                Search term and Search parameter                 ---*/
  typedef struct {
    BYTE1          type;                     /*                            */
                        /*--  Search term string --*/
                        #define WORD_WORD      'W'
                        /*--  Back reference filename (full path)  --*/
                        #define WORD_BACKREF   'B'
                        #define WORD_WHOLE_DOC 'D'
    BYTE1          start_option;
                        #define FROM_ANYWHERE        0x00
                        #define FROM_BOUNDARY        'S'
    BYTE1          casesens;
                   /*--  Case sensitive (Alphabet upper | lower )   --*/
                   #define  CASE_SENSITIVE     'C'
                   /*--  Case insensitive (Alphabet upper | lower ) --*/
                   #define  NOT_CASE_SENSITIVE 'N'
/*  BYTE1          res1[1];                                       @STEM_D  */
    UBYTE1         stemming;                                   /* @STEM_A  */
                   #define STEMMING_ENGLISH 1
                   #define STEMMING_YES     STEMMING_ENGLISH
                   #define STEMMING_NONE     0
    UBYTE1        *wordp;            /* Search term (back reference fname) */
    INT2           wordlen;          /* Length of Search term (backref fn) */
    INT2           precision;        /* Matching level  1 % - 100 %        */
    /*---------------          To specify Range            ----------------*/
    INT4           rangenum;         /* Number of pairs of block number    */
                                     /*   0 : Not specified                */
                                 #define EVEN_BLOCK_RANGE (-1)  /* @4-228C */
                                     /*  -1 : All even block number        */
    RTRVRANGE     *range;            /* Range (Pairs of start and end)     */
    INT2           range_dlmno;      /* Separating-rule number             */

    /*---------------   To specify Range by Field number  --------- @FLD-A */
    UINT2          fld_number;       /* Field nubmer (0: Not specified)    */
    BYTE1          res3[8];
    /*------------------------------------------------------------- @FLD_A */
    BYTE1          mask[2];          /* Mask characters                    */
    BYTE1          res5[2];
    INT2           weight;           /* Weight if WEIGHT_SPECIFIED         */
    BYTE1          res6[2];
    INT4           docfreq;          /*                                    */
    INT4           search_flag;      /* Options                            */
              #define PREFER_WILDCARD  1
              /* ON:  Wildcard search is done when both wildcard           */
              /*      and fuzzy are specified.                             */
              /* OFF: Fuzzy search is done when both are specified         */
    BYTE1          res7[12];         /*                                    */
/*  BYTE1          res7[16];                                               */
  } RTRVWORD;


  /*---                      Search Operator                            ---*/
  typedef struct {
    BYTE1   op;                              /* Operator                   */
                 #define OP_AND               'A'
                 #define OP_OR                'O'
                 #define OP_IN_SAME_BLOCK     'D'
                 #define OP_NOT               'N'

                 #define OP_IN_SAME_DELIMITER  OP_IN_SAME_BLOCK
    BYTE1   rsv0[1];        /*  (reserved)                                 */
    INT2    op_dlmno;       /* Separating-rule number                      */
                            /*   used only when op = OP_IN_SAME_BLOCK      */
    INT4    discon_count;   /* > 0 : Maximum number of occurrences of      */
                            /*       total result                          */
                            /* < 0 : Maximum number of documents of total  */
                            /*       result                                */
                            /* = 0 : Never discontinued                    */
                            /* If discontinued, retrieve_status field of   */
                            /* GTRSTATUS is set to RETRIEVE_DISCONTINUED.  */
                            /*  0 : Never discontinued                     */
    BYTE1   rsv1[28];       /*  (reserved)                                 */
  } RTRVOP;


  /*--   Search parameters for GTRsimpleSearch  --*/
  typedef struct {
    RTRVOP    Operator;            /* Operator                             */
    BYTE1     casesens;            /* ALpha upper/lower case sensitivity   */
              /*--  Case sensitive              --*/
              /*  #define  CASE_SENSITIVE     'C'                          */
              /*--  Case insensitive            --*/
              /*  #define  NOT_CASE_SENSITIVE 'N'                          */
    BYTE1     rsv0[1];
    INT2      precision;           /* Matching level ( 1% - 100% )         */
    INT4      rangenum;            /* Number of range pairs                */
    RTRVRANGE *range;              /* Array of range pairs                 */
    INT2      range_dlmno;         /* Separating-rule number               */
    BYTE1     start_option;        /* For Hangul only                      */
              /*     #define FROM_ANYWHERE        0x00                     */
              /*     #define FROM_BOUNDARY        'S'                      */
/*  BYTE1     rsv1[1];                 (reserved)                  @STEM_D */
    UBYTE1        stemming;        /*                              @STEM_A */
                  /*  #define STEMMING_ENGLISH  1                          */
                  /*  #define STEMMING_YES     STEMMING_ENGLISH            */
                  /*  #define STEMMING_NONE     0                          */
    BYTE1     mask[2];             /* mask[0] : mask char 1                */
                                   /* mask[1] : mask char 2                */
    UINT2          fld_number;   /* Field nubmer (0: Not specified) @FLD_A */
    INT4      search_flag;         /* Options                              */
              /* #define PREFER_WILDCARD  1                                */
              /*   ON:  Wildcard search is done when both wildcard         */
              /*        and fuzzy are specified.                           */
              /*   OFF: Fuzzy search is done when both are specified       */
    BYTE1     rsv2[24];            /*  (reserved)                          */
/*  BYTE1     rsv2[28];                (reserved)                          */
  } SIMPLERTRVOPT;


  /*---   answer_number_of_occ Occ's are arranged in the result file    ---*/
  typedef struct {
    INT4           doc;                      /* Document number            */
    INT4           pos;                      /* Hit Start position number  */
    UINT2          num;                      /* Number of hit characters   */
    UBYTE1         percent;                  /* Matching level of hit      */
    UBYTE1         word_rule;                /* word rule                  */
    INT2           wordNO;                   /*  (reserved)                */
    INT2           blockNO;                  /*  (reserved)                */
  } OCC;


  /*---   As a result of Ranking   ---*/
  typedef struct {
    INT4           doc;         /* Document number                         */
    UINT4          score_func;  /* Score assigned to this doc by callback @4-005a */
    FLOAT4         score;       /* Score assigned to this doc by ranking   */
    INT4           occstart;    /* Start Occ No. of occ for this document  */
    INT4           occnum;      /* Number of occurrences for this document */
  } RANKDOC;

  typedef UINT4 (*GTRRankFunc)(UINT4 ulDocID, void *pUserData); /* @4-005a */

  typedef struct {                                              /* @4-005a */
    GTRRankFunc   entry;                                        /* @4-005a */
    void          *pUserData;                                   /* @4-005a */
    UBYTE1        rank_order;                                   /* @4-005a */
                  #define GTR_DESCENDING            0x00        /* @4-005a */
                  #define GTR_ASCENDING             0x80        /* @4-005a */
    BYTE1         rsv0[23];                                     /* @4-035A */
  } GTRRANKFUNCINFO;                                            /* @4-005a */

  /*---   Parameters for Ranking  ---*/
  typedef struct {
    INT4          rankcount;      /* Max. number of documents to be ranked */
    RANKDOC       *rankdoc;       /* Document list as a result of ranking  */
    UBYTE1        weight_option;  /* how to rank                           */
                  #define WEIGHT_SPECIFIED          0x01
                  #define WEIGHT_FUNC               0x02        /* @4-005a */
                  #define WEIGHT_FREQUENCY          0x10
                  #define WEIGHT_STATISTICS         0x20
                  #define WEIGHT_DEFAULT            0x30
                  #define WEIGHT_ALL                0x31
    BYTE1         howto_operate;  /* fuzzy AND/NOT  or  strict AND/NOT     */
                  #define OPERATE_STRICT 'S'
                  #define OPERATE_FUZZY  'F'
                  #define OPERATE_DEFAULT OPERATE_STRICT
/*  BYTE1         dummy[14];         **  (reserved)                @4-005d */
    BYTE1         rsv0[2];           /*  (reserved)                @4-035A */
    GTRRANKFUNCINFO *pRankFuncInfo;  /* user exit for rank value   @4-035A */
    /* GTRRANKFUNCINFO *rankfuncinfo; */ /* user exit for rank value   @4-005a */ /* @4-035D */
    BYTE1         dummy[16];        /*  (reserved) 10->16 @4-035C  @4-005a */
  } RTRVRANK;


  /*--  HIT information  --*/
  typedef struct {                       /* HIT information                */
    INT4 startoffset;                    /* Start position (offset in doc) */
    INT4 endoffset;                      /* End   position (offset in doc) */
  } HITOFFSET;


  /*--   Reserved characters for GTR's format query (for high)  --*/
  #define GTR_AND_MARK      '*'
  #define GTR_OR_MARK       '+'
  #define GTR_NOT_MARK      '-'
  #define GTR_RANGE_MARK    '@'
  #define GTR_CASE_MARK     '#'
  #define GTR_PERCENT_MARK  '%'
  #define GTR_WEIGHT_MARK   '$'
  #define GTR_SNOT_MARK     '!'

  /*--      reserved   --*/
  typedef struct {
    UBYTE1        *impwordp;
    INT4           impwlen;
    UBYTE1         impwprio;
    UBYTE1         impwprec;
    INT2           impwweight;
  } IMPWORD;

  /*--      reserved   --*/
  typedef struct {
    INT2  fieldNO;
    BYTE1 res0[2];
    INT4  low_num;                   /*          >=                   */
    INT4  high_num;                  /*          <=                   */
    BYTE1 res1[12];
  } RTRVNUMERIC;

  /*--      reserved   --*/
  typedef struct {
    INT2          fieldNO;
    BYTE1         rsv0[2];
    UBYTE1        *string_ptr;


    INT4          string_len;
    BYTE1         mask[2];

    BYTE1         rsv1[2];
    INT2          precision;
    UBYTE1        casesens;
    BYTE1         rsv2[17];
  } RTRVFIELD;
  /*---  Max. FieldNO  ---*/
  #define FIELDNOMAX  254

  /*---  Min. FieldNO  ---*/
  #define FIELDNOMIN    0


  /*---     reserved    ---*/
  typedef struct {
    INT4 doc;
    INT4 size;
    INT4 reserved1;
    INT4 reserved2;
    INT4 num;
  } RINF_DOC;

  /*---     reserved    ---*/
  typedef struct {
    INT4   wordNO;
    FLOAT4 freq;
  } RINF_FRQ;


  /*---   For Item Search  ---*/
  typedef struct {
    INT2          item_number;
    BYTE1         item_type;
                  /* #define ITEMTYPE_CHAR    'C'   @3-063D defined in REGISTITEM */
                  /* #define ITEMTYPE_NUM     'N'   @3-063D defined in REGISTITEM */
                  #define ITEMTYPE_BACKREF 'B'
                  #define ITEMTYPE_NUMSET  'S'
    BYTE1         rsv0[9];

    /*-----  Character Field  -----*/
    UBYTE1        *char_ptr;
    INT2          char_len;
    BYTE1         char_mask[2];
    INT2          char_precision;
    BYTE1         char_casesens;
    BYTE1         rsv1[17];

    /*-----  Numeric Field    -----*/
    FLOAT8        num_min_value;
    FLOAT8        num_max_value;
    BYTE1         num_min_equal;
                  #define INCLUDE_EQUAL 'I'
                  #define EXCLUDE_EQUAL 'X'
    BYTE1         num_max_equal;
    BYTE1         rsv2[18];

    /*-----  Back reference   -----*/
    BYTE1         *bref_fname;
    INT2          bref_fnamelen;
    BYTE1         rsv3[10];

    /*-----  Numeric Set  -----*/
    INT4          numset_num;
    FLOAT8        *numset_value;
  } SEARCHITEM;

  /*--- Struct for Mix Search ---*/     /* @4-001A */
  typedef union {                       /* @4-001A */
    RTRVWORD      word;                 /* @4-001A */
    SEARCHITEM    item;                 /* @4-001A */
  } GTRMIXTERM;                         /* @4-001A */ /* Add 'GTR' @4-035C */

  typedef struct {                      /* @4-001A */
    BYTE1         mix_type;             /* @4-001A */
    BYTE1         rsv0[3];              /* @4-035A */
    GTRMIXTERM    term;                 /* @4-001A */
    BYTE1         rsv1[24];             /* @4-035A */
  } GTRSEARCHMIX;                       /* @4-001A */ /* Add 'GTR' @4-035C */

  typedef INT4  * GTRRESULT_HANDLE;
  typedef INT2  * GTRCONVERT_HANDLE;

  /*--- Skipping Callback ---*/                                                                      /* @4-006A */
  /* GTRSKIPDATA: Skipping callback data */                                                          /* @4-006A */
  typedef struct {                                                                                   /* @4-006A */
    UINT4  lDocNo ;                     /* Corrent document number       (in)  */                    /* @4-006A */ /* @4-047C ulDocID -> lDocNo */
    UINT4  ulSkipOrder;                 /* Skipping order                (in)  */                    /* @4-006A */
    UINT4  lNextDocNo;                  /* Next document number          (out) */                    /* @4-006A */ /* @4-047C ulNextDocID -> lNextDocNo */
    UBYTE1 ucRequired;                  /* Is current document required? (out) */                    /* @4-006A */
    BYTE1  rsv0[19];                    /* (reserved)                          */                    /* @4-035A */
  } GTRSKIPDATA;                                                                                     /* @4-006A */
                                                                                                     /* @4-006A */
  /* Skipping callback function */                                                                   /* @4-006A */
  typedef INT4 (*GTRSkipFunc)(GTRSKIPDATA *, void *);                                                /* @4-006A */
                                                                                                     /* @4-006A */
  /* GTRSKIPFUNCINFO: Information on skipping callback function */                                   /* @4-006A */
  typedef struct {                                                                                   /* @4-006A */
    GTRSkipFunc  entry;                /* Pointer to callback function */                            /* @4-006A */
    UINT4        ulSkipOrder;          /* Skipping order               */                            /* @4-006A */
    void         *pUserData;           /* Pointer to user data         */                            /* @4-006A */
    BYTE1        rsv0[20];             /* (reserved)                   */                            /* @4-006A */
  } GTRSKIPFUNCINFO;                                                                                 /* @4-006A */
                                                                                                     /* @4-006A */
  /*--- Search order ---*/                                                                           /* @4-006A */
  #define GTR_SEARCHORDER_ASCENDING       0x1   /* Ascending search    */                            /* @4-006A */

  /*-----------                              ------- @V3_A */
  /*-----------   Search Result Parameters   ------- @V3_A */
  /*-----------                              ------- @V3_A */
  typedef struct {
    UBYTE1           res_type;                      /* Type of result  I  */
                     #define GTR_RES_FILE               1
                     #define GTR_RES_HANDLE             2
                     #define GTR_RES_FILE_WITH_INFO     4                                           /* @4-035A */
                     #define GTR_RES_NONE               255
/*                     #define GTR_RES_FILE_FAST 4                               */   /* @4-001A */ /* @4-035D */
    UBYTE1           res_dataform;           /* Data format of result   I */
                     #define GTR_RES_OCCDATA  1
                     #define GTR_RES_ONLY_DOC_INFO   2   /* @4-001A */                              /* @4-001A */
    BYTE1            rsv1[13];
    UBYTE1           result_status;          /* Status of Result        O */
                     #define GTR_RESULT_SORTED   0x80
                     #define GTR_RESULT_DOCONLY  0x40                                               /* @4-001A */
    INT4             result_docnum;          /* Number of Documents     O */
    INT4             result_occnum;          /* Number of Occurrences   O */
    BYTE1            rsv2[20];
    /*---  Specify  only when res_type = GTR_RES_FILE    ---*/
/*     BYTE1            res_fname[DIRLENMAX+FNAMELENMAX]; */    /* @4-220D */
    BYTE1            res_fname[EXTFNAMELENMAX];                 /* @4-220A */
                                   /* Search result filename (fullpath) I */
    BYTE1            rsv3[8];

    /*---  Returned only when res_type = GTR_RES_HANDLE  ---*/
    GTRRESULT_HANDLE res_handle;         /* Search result handle        O */
    INT4             res_handle_memory;  /* Memory size to be allocated I */

    INT4      res_wordnum_max;   /* array size of discon_count_word                                *//* @3-048A */
    INT4      result_wordnum;    /* Num of words which consists of complex query. Returned to appl *//* @3-048A */
    INT4*     result_hits_word;  /* Num of hits occurrences per word.                              *//* @3-048A */
    BYTE1            rsv4[52];   /* Changed from [64]                                              *//* @3-048C */
  } GTRSEARCH_RES;


  typedef signed char * GTRSEARCH_HANDLE;
  typedef RTRVRANK GTRSEARCH_RANK;
  typedef void * GTRKEYITERATOR;                                           /* @4-004a */
  typedef void * GTRPOSITERATOR;                                           /* @4-010a */

  typedef struct {                                                                                   /* @4-001A */
    INT4          head;         /* Value must be 0xFFFFFFFF */                                       /* @4-001A */
                  #define GTR_BACKREF_DLM         0xFFFFFFFF                                         /* @4-001A */
    INT4          docnum;                                                                            /* @4-001A */
    INT4          occnum;                                                                            /* @4-001A */
    BYTE1         status;         /* Specify the type of data */                                     /* @4-001A */
    /*--- 'status' field represents the status of back referrence file as follows: ---*/             /* @4-001A */
    /*---         #define GTR_RESULT_SORTED   0x80                                 ---*/             /* @4-001A */
    /*---         #define GTR_RESULT_DOCONLY  0x40                                 ---*/             /* @4-001A */
    /*---  (These macros have already been defined above.)                         ---*/             /* @4-001A */
    BYTE1         rsv[115];     /* reserved                 */                                       /* @4-001A */
  } GTRBACKREFHEAD;                                                                                  /* @4-001A */

  /*-----------                                 ------- @V3_A */
  /*-----------   Search Condition Parameters   ------- @V3_A */
  /*-----------                                 ------- @V3_A */
  typedef struct {
    GTRSEARCH_RES  *scope; /* Specify result of previous search as scope  */
                           /* NULL : Scope is whole index                 */
    INT4           discon_count;               /* Discontinued count      */
                                               /* 0 : Never disconotinued */
    INT4           mask_discon_count;          /*                         */
    GTRSKIPFUNCINFO *pSkipFuncInfo;            /* Skip callback function info */                     /* @4-006A */
    UBYTE1         codepage;                   /* same as GTRHCREATEPARM  */
    UBYTE1         query_type;                 /* Type of query           */
                   #define GTR_TEXT_COND_SIMPLE   1
                   #define GTR_TEXT_COND_QUERY    2
                   #define GTR_TEXT_COND_STRUCT   3
                   #define GTR_ITEM_COND_STRUCT   4
                   #define GTR_MIX_COND_STRUCT    5     /* @4-001A */                                /* @4-001A */
                   #define GTR_MIX_COND_QUERY     6                                                  /* @4-014A */

    BYTE1          rsv0[10];                   /* (reserved)              */                         /* @4-006A */
    /* BYTE1          rsv0[14];                  */ /* (reserved)              */                    /* @4-006D */

    /*---  Specify only when query_type = GTR_TEXT_COND_SIMPLE  ---*/
    UBYTE1        *simple_query;   /* Search term delimited by blank      */
    SIMPLERTRVOPT simple_option;   /* Search parameters and an operator   */
    BYTE1         rsv1[8];         /*  (reserved)                         */

    /*---  Specify only when query_type = GTR_TEXT_COND_QUERY   ---*/
    UBYTE1        *query;          /* Query with GTR's format             */
    BYTE1         rsv2[8];         /*  (reserved)                         */

    /*---  Specify only when query_type = GTR_TEXT_COND_STRUCT  ---*/
    INT4          struct_num;      /* Number of search terms              */
    RTRVWORD      *struct_word;    /* Search term and search parameters   */
    BYTE1         *struct_order;   /* Query with search term No.          */
                                   /* (Ex.)  1 * ( 2 + 3 )                */
    BYTE1         *frq_fname;      /*                                     */
    BYTE1         rsv3[8];         /*  (reserved)                         */

    /*---  Specify only when query_type = GTR_ITEM_COND_STRUCT  ---*/
    INT4          item_num;        /* Number of item                      */
    SEARCHITEM    *item;           /* Conditions                          */
    BYTE1         *item_order;     /* Query with item No.                 */
                                   /* (Ex.)  1 * ( 2 + 3 )                */

    INT4       discon_count_word;  /* discon_count of occ for intermediate */
    /*---  Specify only when query_type = GTR_ITEM_COND_STRUCT  ---*/                                /* @4-001A */
    INT4          mix_num;         /* Number of mix struct                */                         /* @4-001A */
    GTRSEARCHMIX  *mix;            /* Search term and search parameters   */                         /* @4-001A */
    BYTE1         *mix_order;      /* Query with item No.                 */                         /* @4-001A */
                                   /* (Ex.)  1 * ( 2 + 3 )                */                         /* @4-001A */
    BYTE1         rsv4[48];        /*  (reserved)                          */                        /* @4-001A */
/*     BYTE1         rsv4[60];         */ /*  (reserved)                          */                 /* @4-001D */
  } GTRSEARCH_COND;


/*------------------------------------------*/
/*              For Verify                  */
/*------------------------------------------*/
  /*---  Index document information for high  ---*/
  typedef struct {
    INT4 mindoc;         /* Min. document number in the main index         */
    INT4 maxdoc;         /* Max. document number in the main index         */
    INT4 docnum;         /* Number of documents  in the main index         */
    INT4 deltamindoc;    /* Min. document number in the supplemental index */
    INT4 deltamaxdoc;    /* Max. document number in the supplemental index */
    INT4 deltadocnum;    /* Number of documents  in the supplemental index */
    char reserved[32];
  } INDEXDOCINFO;

/*------------------------------------------*/
/*             For GetIndxInfo              */
/*------------------------------------------*/
  typedef struct {
    UBYTE1        index_purpose;   /* Specify index type to be referred I */
                  /* #define TEXT_SEARCH   0                              */
                  /* #define ITEM_SEARCH   3                              */
    BYTE1         res0[79];
  } GTRGETINFO;

/*------------------------------------------*/
/*             For GetKeyInfo       @4-003  */
/*------------------------------------------*/
  typedef struct {
    UBYTE1        *pszWord;             /* pointer to word string         */
    INT4          lNumKeyRec;           /* the number of KEY_REC          */
    UBYTE1        ucCodepage;           /* codepage                       */
    BYTE1         res0[23];             /* reserved                       */
  } GTRKEYINFO;

/*------------------------------------------*/
/*      For API of Batch-Insert             */
/*------------------------------------------*/
typedef struct {
   char*             docptr;
   INT4              doclen;
   INT4              docno;
   BOUNDARYINFO      boundaryinfo;
} GTRDOCINFO;

typedef struct {
   INT4              docno;
   INT4              registitem_num;
   REGISTITEM*       registitem;
} GTRITEMINFO;


/*************************************************************************/
/*             Return Codes set in retcode field of GTRSTATUS            */
/*************************************************************************/
  #define INVALID_HANDLE              1
  #define PARAMETER_ERROR             2
  #define FILE_OPEN_ERROR             3
  #define FILE_READ_ERROR             4
  #define FILE_WRITE_ERROR            5
  #define FILE_SEEK_ERROR             6
  #define FILE_TELL_ERROR             7
  #define FILE_REMOVE_ERROR           8
  #define FILE_RENAME_ERROR           9
  #define FILE_CLOSE_ERROR           10
  #define MEMORY_ERROR               11
  #define SEQUENCE_ERROR             15
  #define INTEGRITY_ERROR            16
  #define UNEXPECTED_ERROR           17
  #define INDEX_BASIC_ERROR          18
  #define END_OF_FILE                19
  #define INDEX_NOT_EXIST            20
  #define INVALID_CALL               21
  #define INVALID_BACKREF            22
  #define VERSION_ERROR              23
  #define INVALID_DELIMITER_NO       24
  #define NO_QUERY_ERROR             25
  #define PRECISION_ERROR            26
  #define RANGE_ERROR                27
  #define OPERATOR_ERROR             28
  #define INDEX_NAME_ERROR           29
  #define TOO_BIG_DOCNO_ERROR        30
  #define TOO_BIG_DOC_ERROR          31
  #define START_OPTION_ERROR         32
  #define START_POSITION_ERROR       34
  #define NAME_LENGTH_ERROR          35
  #define NOT_SUPPORTED_YET          50

  #define NORMALIZE_ERROR           101
  #define QUERY_SYNTAX_ERROR        102
  #define NEST_ERROR                103
  #define OP_SEQUENCE_ERROR         104
  #define QUOTE_SYNTAX_ERROR        105
  #define CASE_OPTION_ERROR         106
  #define BACKREF_ERROR             107
  #define CODEPAGE_ERROR            108
  #define BOUNDARY_SEQUENCE_ERROR   109
  #define OCC_SEQUENCE_ERROR        110
  #define MASK_CHAR_ERROR           111
  #define WEIGHT_ERROR              112
  #define RANKING_BACKREF_ERROR     113
  #define FIELDNO_ERROR             114
  #define ITEM_NUMBER_ERROR         114
  #define RESERVED_PARM_ERROR       115
  #define ITEM_PARM_ERROR           116
  #define ITEM_TYPE_ERROR           117
  #define ITEM_LENGTH_ERROR         118
  #define NUMBER_OF_ITEM_ERROR      119
  #define FLD_INFO_ERROR            120

  #define ENVIRONMENT_ERROR         122
  #define END_OF_RESULT             123
  #define START_OF_RESULT           124
  #define HANDLE_OPENED_ERROR       125
  #define HANDLE_NOT_OPENED_ERROR   126
  #define SCOPE_ERROR               127
  #define RESERVED_MEMBER_ERROR     128
  #define TOO_BIG_WORK_AREA_ERROR   129
  #define TOO_MANY_HITS_ERROR       130

  /*----- @4-002A Start -----*/
  #define UNSUPPORTED_FUNC_REQUEST  131
  #define REQUEST_ERROR             132
/*  #define ALREADY_DOCID_EXIST       133 */
  #define CONFLICTION               134
  #define ILLEGAL_REQUEST           135

  #define MKDIR_ERROR               140
  #define RMDIR_ERROR               141
  /*----- @4-002A End   -----*/

  #define SKIPCB_ERROR              150                      /* @4-006A */
  #define STOPWORD_DEFINE_ERROR     151                      /* @4-038A */
  #define UBM_ERROR                 152                      /* @4-002A */
  #define ALREADY_DOCID_EXIST       153                      /* @4-050A */
  #define TOO_MUCH_POSITION_INFO    154                      /* @4-212A */
  #define EXPIRED_ERROR             999                      /* @TIMEBOMB */

  #define GTR_USER_RETCODE        10000

/*************************************************************************/
/*   Return Codes set in retcode of Batch Insert API                     */
/*      GTRcreateIndexPluralDoc                                          */
/*      GTRcreateItemIndexPluralDoc                  @3-030              */
/*************************************************************************/
  #define GTR_PLURALNORMALEND            0
  #define GTR_PLURALERROR_ONE           (-1)                  /* @4-228C */
  #define GTR_PLURALERROR_INVALIDPARA   (-3)                  /* @4-228C */

  /*---------------------------------------------------------------------*/
  /*------------------------  For Compatibility  ------------------------*/
  #ifdef GTR_V1_MIG
    #define blockend_num  boundary_num
    #define blockend_ptr  boundary_ptr
    #define BLOCKENDINFO  BOUNDARYINFO
    #define BLOCK_END_SEQUENCE_ERROR  BOUNDARY_SEQUENCE_ERROR
  #endif
  #define FIELD_SEARCH ITEM_SEARCH                                                                   /* @4-002A */
  #define NUMERIC_SEARCH ITEM_SEARCH                                                                 /* @4-002A */
/*   #define FIELD_SEARCH    1 */                                                                    /* @4-002D */
/*   #define NUMERIC_SEARCH  2 */                                                                    /* @4-002D */

/*************************************************************************/
/* changes for function callbacks                                @3-073A */
/*************************************************************************/

  #ifdef GTR_CALLBACKS                                          /* @3-073A */

    typedef struct tag_FTGCallbackStream                        /* @3-073A */
    {                                                           /* @3-073A */
      void * (*fpMalloc)  (size_t );                            /* @3-073A */
      void   (*fpFree)    (void *);                             /* @3-073A */
      void * (*fpRealloc) (void *, size_t);                     /* @3-073A */
      UINT2   (*fpPreempt) (void);                              /* @3-073A */
      void (*fpProcThreadID) (unsigned long*, unsigned long*);  /* @4-250A */
    }                                                           /* @3-073A */
    FTGCallbackStream;                                          /* @3-073A */

/* --- @4-250D start --- */
/*    #ifndef GTR_MAC don't use these on Mac -- always use consistent memory calls  @3-073A */
/*      void *FTG_malloc( size_t size );                   */       /* @3-073A */
/*      void FTG_free( void *ptr );                        */       /* @3-073A */
/*      void *FTG_realloc( void *ptr, size_t size );       */       /* @3-073A */
/*      #define malloc(sz) FTG_malloc(sz)                  */       /* @3-073A */
/*      #define free(ptr) FTG_free(ptr)                    */       /* @3-073A */
/*      #define realloc(ptr, size) FTG_realloc(ptr, size)  */       /* @3-073A */
/*    #endif                                               */       /* @3-073A */

/*    void FTG_preempt( GTRSTATUS *gsp );                  */     /* @3-073A */
/* --- @4-250D end --- */
    #define TIME_TO_PREEMPT 500                                 /* @3-073A */
  #endif                                                        /* @3-073A */

/*---  4-byte boundary  ---*/
#ifdef _WIN32
#pragma pack(pop)
#endif
#endif
