//  Source File Name: NXSample.java
//
//**************************************************************************
//* Licensed Materials - Property of IBM                                   *
//* 5648-D44                                                               *
//* (c) Copyright IBM Corporation 1999.                                    *
//* All rights reserved.                                                   *
//*                                                                        *
//* US Government Users Restricted Rights - Use, duplication or            *
//* disclosure restricted by GSA ADP Schedule Contract with                *
//* IBM Corporation.                                                       *
//**************************************************************************
//*  Permission Notice                                                     *
//*                                                                        *
//*  Permission is granted to copy, use, modify, and merge this sample     *
//*  software into your applications and to permit others to do any of the *
//*  foregoing. You may further distribute this software for               *
//*  commercial purposes only as part of your application that adds        *
//*  significant value and function beyond that provided by these          *
//*  samples.                                                              *
//*  You must include this permission statement and retain the copyright   *
//*  notice in all copies and modified versions of this software.          *
//*                                                                        *
//**************************************************************************
//*                                                                        *
//*  DISCLAIMER OF WARRANTIES                                              *
//*                                                                        *
//*  The sample software is provided to you by IBM to assist you in        *
//*  developing your applications. THIS SOFTWARE IS PROVIDED AS-IS,        *
//*  WITHOUT WARRANTY OF ANY KIND. IBM SHALL NOT BE LIABLE FOR ANY         *
//*  DAMAGES ARISING OUT OF YOUR USE OR THE USE BY ANY THIRD PARTY OF      *
//*  OF THE SAMPLE SOFTWARE EVEN IF IT HAS BEEN ADVISED OF THE POSSIBILITY *
//*  OF SUCH DAMAGES. IN ADDITION, IBM SHALL NOT BE LIABLE FOR ANY THIRD   *
//*  PARTY CLAIMS AGAINST YOU.                                             *
//*                                                                        *
//**************************************************************************
//   Sample program - Using DB2 Net Search Extender
//                    in a Java program

//  Run these samples by the following steps:
//  (2) run the txfpsamp program to create a sample database and a sample table
//  (3) compile this Java file (javac NXSampleRank.java)
//  (4) run the sample (java NXSampleRank yourDatabase [ <username> <password> ] <search term>)

//   NOTES: (1) The CLASSPATH and shared library path environment variables
//              must be set, as for any JDBC application.
//          (2) Visit http://www.software.ibm.com/data/db2/java
//              for current DB2 Java information

//  For more information on this sample, refer to the "Searching in a database"
//  section of the DB2 Net Search Extender Guide.

//  For more information on setting up your DB2 Java development environment and
//  running Java programs, refer to the "Building Java Applets and Applications"
//  section in the Building Applications for UNIX** Environments Guide or the
//  Building Applications for Windows and OS/2 Environments Guide.

//  For more information on the SQL language, refer to the SQL Reference.


import java.sql.*;	// SQLJ classes
import java.lang.*;
import java.io.*;

public class NXSampleRank {

public static void main(java.lang.String[] argv) {
  try
    {
      System.out.println ();
      System.out.println ("DB2 Net Search Extender Java search sample");

      Class.forName ("COM.ibm.db2.jdbc.app.DB2Driver").newInstance ();
    }
  catch (Exception e)
    {
      System.out.println ("\nError loading DB2 Driver.\n" + e.toString() + "\n");
      System.out.println ("Ensure that the CLASSPATH variable points to the files sqlj.zip, db2java.zip, and runtime.zip.");
      System.out.println ("Ensure that the CLASSPATH variable points to the directory where NXSampleRank.class resides.\n");
    }	

  Connection con = null;
  try
    {
      // parameter values for the call of the "textSearch" stored procedure:

	// searchTerm: The word or words to be searched for. (see documentation for syntax)
	String searchTerm = "";	

   	// maxHitCount: maximum number of hits, set to "0" for unlimited
	int maxHitCount = 32000;

	// maxIntermediateHitCount: maximum number of hits for each search term, set to "0" for unlimited
	int maxIntermHitCount = 32000;

        // startRow: row number of the first result of the result buffer to be put into the result set
	int startRowNumber = 0;

        // maxRowsToReturn: maximum number of results in the result set of the stored procedure
	int maxRowsToReturn = 100;

	// indexname: name of the index (this is the id-name you specified in the enable text column command)
	String indexname = "comment";

	// indexDirectory: directory where the index files are stored. <null> means that the default directory $DB2NX_INSTOWNERHOMEDIR/db2nx/indices/<database name>/<index name> is used.
	String index_directory = null;

	// tmpDirectory: directory where temporary files can be written by the stored procedure
	String tmp_directory = null;

	// sqlStatement: An SQL statement that specifies the columns and rows of the result table returned by the stored procedure.
	String sqlStatement = "";  // not necessary since we will get the results from in-memory table

        // rankOper: (0=no ranking , 1=rank opration type fuzzy , 2=rank opration type strict)
	int rankOper = 1;

        // dataSource: (0=results come from in-memory table, 1=results come from DB2)
	int dataSource = 0;


	// information about the DB2 Net Search Extender stored procedure:
	String storedProcName = "desfpssp!textSearch_r";	// <library name>!<stored procedure name>

      // parameter variables

	String dbname = "";  	  // database name or alias

	// read command line parameters

	String userid = null;
	String passwd = null;

	if ( argv.length == 4 )
	  {
            dbname = new String( argv[0] );
            userid = new String( argv[1] );
            passwd = new String( argv[2] );
	    searchTerm = new String( argv[3] );
	  }
	else if ( argv.length == 2 )
	  {
            dbname = new String( argv[0] );
	    searchTerm = new String( argv[1] );
	  }
	else
	  {
	    System.out.println();
	    System.out.println("usage:   java NXSampleRank yourDatabase [ <user-id> <password> ] <search term>");
	    System.out.println();
	    System.out.println("example: java NXSampleRank yourDatabase \"\\\"bestseller\\\" & \\\"pulitzer\\\"\"");
	    System.out.println();
	    System.exit(0);
	  }


	// connect to the database

	if ( userid == null )
	  {
 	    // connect with default id/password
	    // the prefix "jdbc:db2:" is obligatory

  	    con = DriverManager.getConnection("jdbc:db2:" + dbname);
	  }
	else
	  {
	    // connect with user-provided username and password

  	    con = DriverManager.getConnection("jdbc:db2:" + dbname, userid, passwd);
	  }

	// call the stored procedure

	NXSampleRank.callStoredProc	(con,
            storedProcName,
	    searchTerm,
  	    maxHitCount,
	    maxIntermHitCount,
	    startRowNumber,
	    maxRowsToReturn,
	    indexname,
	    index_directory,
	    tmp_directory,
            rankOper,
	    dataSource,
	    sqlStatement);
	
	// close connection to the database

 	con.close ();	

    }
    catch (Exception e)
      {
        try
	  {
            // close connection to the database if an error has occured
	    con.close();	
 	  } catch (Exception x)
	  {
            e.printStackTrace ();
	  }
      }

}
	
public static void callStoredProc (Connection con,
 String name,
 String searchTerm,
 int maxHitCount,
 int maxIntermHitCount,
 int startRowNumber,
 int maxRowsToReturn,
 String indexname,
 String index_directory,
 String tmp_directory,
 int rankOper,
 int dataSource,
 String sqlStatement) throws Exception
{
    // print the parameter settings

    System.out.println("Search parameters:");
    System.out.println();
    System.out.println("index name:         " + indexname);
    System.out.println("search term:        " + searchTerm);
    System.out.println("sql query:          " + sqlStatement);
    System.out.println("max hit count:      " + maxHitCount);
    System.out.println("max intermediate:   " + maxIntermHitCount);
    System.out.println("start row number:   " + startRowNumber);
    System.out.println("max rows to return: " + maxRowsToReturn);
    System.out.println("rank operation type:" + rankOper);
    System.out.println("data source:        " + dataSource);
    System.out.println();
    System.out.println("Search results:");

    //  prepare the CALL statement

    CallableStatement stmt;
    String sql = "Call " + name + "(?,?,?,?,?,?,?,?,?,?,?,?,?) ";
    stmt = con.prepareCall (sql);

    //  register the output parameters

    stmt.registerOutParameter (1, Types.INTEGER);
    stmt.registerOutParameter (13, Types.CHAR);

    //  set all parameters (input and output)

    int totalDocs = 0;
    String wordCounts = "";
	
    stmt.setInt (1, totalDocs);
    stmt.setString (2, searchTerm);
    stmt.setInt (3, maxHitCount);
    stmt.setInt (4, maxIntermHitCount);
    stmt.setInt (5, startRowNumber);
    stmt.setInt (6, maxRowsToReturn);
    stmt.setString (7, indexname);
    stmt.setString (8, index_directory);
    stmt.setString (9, tmp_directory);
    stmt.setString (10, sqlStatement);
    stmt.setInt (11, rankOper);
    stmt.setInt (12, dataSource);
    stmt.setString (13, wordCounts);

    try
      {
	  //  call the stored procedure

	  stmt.execute ();

	  // the stored procedure "textSearch" returns two output parameters and one result set which contains the "key-column name" entries of the documents found in the index. The "key-column-name" must be the primary index on the database table. It is defined by the USING keyword of the "ENABLE TEXT COLUMN" command.

 	  // define an object to take the result set of the stored procedure
	  ResultSet rs = null;

	  // retrieve contents of the result set first (although the stored procedure "textSearch" is designed to return only one result set, the following code is suitable to read the contents of multible result sets each containing one or more lines)

          int rsCount = 0;

          while( true )
            {
	      // determine if the current result is a result set
	      int rowCount = stmt.getUpdateCount();

	      if( rowCount > 0 )
		{
		    stmt.getMoreResults();
		    continue;
		}
	      if( rowCount == 0 )
		{
		    stmt.getMoreResults();
		    continue;
		}

	      rs = stmt.getResultSet();
	      if( rs != null )
                {
		    System.out.println();
		    rsCount++;
		    ResultSetMetaData stmtInfo = rs.getMetaData();
		    int numOfColumns = stmtInfo.getColumnCount();
		    int r = startRowNumber;	// start counting with the row number given by the startRowNum parameter
		    while( rs.next() )
			{
			    System.out.print("Row " + r + "\t: ");
			    for( int i=1; i <= numOfColumns; i++ )
				{
				    System.out.print(rs.getString(i));
				    if( i != numOfColumns )
			              System.out.print("\t");
				}
			     System.out.println("");
		 	     r++;
			 }					
		    stmt.getMoreResults();
		    System.out.println("");
		    continue;
                }
                break;
            }

          //  retrieve output parameters

	  totalDocs = stmt.getInt(1);
	  wordCounts = stmt.getString(13);
	
	  //  display the information returned by the output parameters of the stored procedure

	  System.out.println ("Documents:   " + totalDocs);
	  System.out.println ();
	  System.out.println ("Documents found per search term: " + wordCounts);

	  stmt.close ();

      } catch(SQLException e) {

 	  if ( e.toString().indexOf("SQL0100W") > 0 )
	      {
		  System.out.println ("\nNo documents were found.");	
	      }
	  else if ( e.toString().indexOf("SQL0438N") > 0 )
	      {
	 	  // retrieve the error message of the message string

		  System.out.println ();
		  System.out.println (e.toString() + "\n");
	      }
	  else
	      {
	 	  System.out.println ("\nAn SQL error occured:\n");
		  System.out.println (e.toString());
	      }
      }

}

}
